@@ -1,10 +1,11 @@
 #!/bin/sh
 
 libtoolize --force --copy
+#gettextize --force --copy --intl
 autoheader
-aclocal-1.7
+aclocal
 autoconf
-automake-1.7 -a -c
+automake -a -c
 
 rm autom4te.cache/*
 rmdir autom4te.cache
