@@ -27,7 +27,7 @@ uint8 Dis6280::GetY(void)
 }
 
 
-static const char *fstrings[18]=
+static const char *fstrings[19]=
 { 
         "#$%02X",       // immediate
         "$%04X",        // RELATIVE(jump)
@@ -37,7 +37,7 @@ static const char *fstrings[18]=
         "$%04X",        //ABS
         "$%04X,X",      // ABS,x
         "$%04X,Y",      // ABS,y
-        "($%04X)",      // IND
+        "($%02X)",      // IND
         "($%02X,X)",    // INX
         "($%02X),Y",    // INY
 	"",		// IMP
@@ -50,8 +50,11 @@ static const char *fstrings[18]=
 	"#$%02X, $%04X, X",  // IMM ABS, X
         "$%02X, $%04X",     // ZP REL
 
+
+	"($%04X)",		// JMP indirect
 };
-static const int flengths[18]={1,1,1,1,1,2,2,2,2,1,1,0, 6, 2, 2, 3, 3, 2};
+
+static const int flengths[19]={1,1,1,1,1,2,2,2,1,1,1,0, 6, 2, 2, 3, 3, 2, 2};
 
 #define IMD(x)	((0<<16)|x)
 #define	REL(x)	((1<<16)|x)
@@ -71,9 +74,10 @@ static const int flengths[18]={1,1,1,1,1,2,2,2,2,1,1,0, 6, 2, 2, 3, 3, 2};
 #define IMDABS(x) ((15 << 16)|x)
 #define IMDABX(x) ((16 << 16)|x)
 #define ZPREL(x) ((17 << 16)|x)
+#define JMPIND(x) ((18 << 16)|x)
 
 typedef struct {
-	char *name;
+	const char *name;
         int type;       /* 1 for read, 2 for write, 3 for r then write. */
 	int32 modes[11];
 } OPS;
@@ -97,7 +101,7 @@ static OPS optable[NUMOPS]=
  {"PLX",1,{IMP(0xFA),-1}},
  {"PLY",1,{IMP(0x7A),-1}},
 
- {"JMP",0,{ABS(0x4C),IND(0x6C),INX(0x7C),-1}},
+ {"JMP",0,{ABS(0x4C),JMPIND(0x6C),INX(0x7C),-1}},
  {"JSR",0,{ABS(0x20),-1}},
 
  {"SAX",0,{IMP(0x22),-1}},
@@ -140,7 +144,7 @@ static OPS optable[NUMOPS]=
 
  {"ASL",1,{IMP(0x0a),ZP(0x06),ZPX(0x16),ABS(0x0E),ABX(0x1E),-1}},
 
- {"DEC",3,{ZP(0xc6),ZPX(0xd6),ABS(0xcE),ABX(0xdE),-1}},
+ {"DEC",3,{IMP(0x3A), ZP(0xc6),ZPX(0xd6),ABS(0xcE),ABX(0xdE),-1}},
 
  {"INC",3,{IMP(0x1A),ZP(0xe6),ZPX(0xf6),ABS(0xeE),ABX(0xfE),-1}},
 
@@ -178,7 +182,7 @@ static OPS optable[NUMOPS]=
  {"ORA",1,{IMD(0x09),ZP(0x05),ZPX(0x15),ABS(0x0D),ABX(0x1d),ABY(0x19),
          IND(0x12),INX(0x01),INY(0x11),-1}},
 
- {"SBC",1,{IMD(0xEB),IMD(0xe9),ZP(0xe5),ZPX(0xf5),ABS(0xeD),ABX(0xfd),ABY(0xf9),
+ {"SBC",1,{IMD(0xe9),ZP(0xe5),ZPX(0xf5),ABS(0xeD),ABX(0xfd),ABY(0xf9),
          IND(0xf2),INX(0xe1),INY(0xf1),-1}},
 
  {"STA",2,{ZP(0x85),ZPX(0x95),ABS(0x8D),ABX(0x9d),ABY(0x99),
@@ -240,7 +244,7 @@ static OPS optable[NUMOPS]=
 
  {"BRA",1,{REL(0x80),-1}},
  
- {"BSR",1,{ABS(0x44),-1}},
+ {"BSR",1,{REL(0x44),-1}},
 
  {"BCC",1,{REL(0x90),-1}},
 
@@ -260,13 +264,12 @@ static OPS optable[NUMOPS]=
 
 };
 
-std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
+void Dis6280::Disassemble(uint16 &a, uint16 SpecialA, char *stringo)
 {
- char stringo[256];
  uint8 buf;
  uint64 arg;
  int32 info;
- int x;
+ int op_index;
  int y;
 
  info=-1;
@@ -274,14 +277,14 @@ std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
  buf=Read(a);
  a++;
 
-  for(x=0;x<NUMOPS;x++)
+  for(op_index=0; op_index < NUMOPS; op_index++)
   {
    y=0;
-   while(optable[x].modes[y]>=0)
+   while(optable[op_index].modes[y]>=0)
    {
-    if((optable[x].modes[y]&0xFF)==buf)
+    if((optable[op_index].modes[y]&0xFF)==buf)
     {
-     info=optable[x].modes[y];
+     info=optable[op_index].modes[y];
      goto endy;
     }
     y++;
@@ -289,17 +292,18 @@ std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
   }
 
   endy:
-  sprintf(stringo,"%02X ",buf);
+  stringo[0] = 0;
   if(info>=0)
   {
    int z=flengths[(info>>16)];
    bool borked = 0;
+   int pow = 0;
+   uint64 arg_orig;
 
-   arg = 0;
+   arg_orig = arg = 0;
 
    if(z)
    {
-    int pow;
     for(pow = 0; pow < z; pow++)
     {
      if(a == SpecialA)
@@ -309,11 +313,9 @@ std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
      }
      uint8 rb = Read(a);
      arg |= (uint64)rb << (pow * 8);
-     sprintf(stringo+strlen(stringo),"%02X ", rb);
      a++;
     }
-    for(; pow < 6; pow++)
-     strcat(stringo, "   ");
+    arg_orig = arg;
     if((info>>16)==1)   /* Relative branch */
      arg = a + (char)arg;
 
@@ -321,9 +323,9 @@ std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
      strcat(stringo, "--------");
     else
     {
-     sprintf(stringo+strlen(stringo),"%s ",optable[x].name);
+     sprintf(stringo+strlen(stringo),"%s ",optable[op_index].name);
 
-     if((info>>16) < 12)
+     if((info>>16) < 12 || (info >> 16) == 18) // ...or JMP indirect
       sprintf(stringo+strlen(stringo), fstrings[info>>16], (unsigned int)arg);
 
         //"$04X, $04X, $04X", // BMT
@@ -358,7 +360,7 @@ std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
 	      sprintf(stringo + strlen(stringo), " @ $%04X = $%02X", (unsigned int)(arg & 0xFF) + 0x2000, Read((arg & 0xFF) + 0x2000));
               break;
       case 2:tmp = arg + 0x2000;
-             if(optable[x].type&1)
+             if(optable[op_index].type&1)
              {
               sprintf(stringo+strlen(stringo),"    @ $%04X",tmp);
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
@@ -366,16 +368,16 @@ std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
              break;
       case 3:tmp=0x2000 + ((arg+GetX())&0xff);
              sprintf(stringo+strlen(stringo),"    @ $%04X",tmp);
-             if(optable[x].type&1)
+             if(optable[op_index].type&1)
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
              break;
       case 4:tmp=0x2000 + ((arg+GetY())&0xff);
              sprintf(stringo+strlen(stringo),"    @ $%04X",tmp);
-             if(optable[x].type&1)
+             if(optable[op_index].type&1)
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
              break;
       case 5:tmp=arg;
-             if(optable[x].type&1) 
+             if(optable[op_index].type&1) 
              {
               sprintf(stringo+strlen(stringo),"  @ $%04X",tmp);
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
@@ -383,45 +385,58 @@ std::string Dis6280::Disassemble(uint16 &a, uint16 SpecialA)
              break;
       case 6:tmp=(arg+GetX())&0xffff;
              sprintf(stringo+strlen(stringo),"  @ $%04X",tmp);
-             if(optable[x].type&1)
+             if(optable[op_index].type&1)
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
              break;
       case 7:tmp=(arg+GetY())&0xffff;
              sprintf(stringo+strlen(stringo),"  @ $%04X",tmp);
-             if(optable[x].type&1)
+             if(optable[op_index].type&1)
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
              break;
-      case 8:
+      case 8: // Indirect
+	     tmp = (arg&0xFF);
+             tmp=Read(0x2000 + tmp) | (Read(0x2000 + ((tmp+1)&0xFF))<<8);
+             sprintf(stringo+strlen(stringo),"  @ $%04X",tmp);
+             if(optable[op_index].type&1)
+              sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
+             break;
+      case 18: // JMP indirect
  	     tmp=Read(arg)|(Read((arg+1)&0xffff)<<8);
              sprintf(stringo+strlen(stringo)," $%04X",tmp);
              break;
       case 9:tmp = ((arg+GetX())&0xFF);
              tmp=Read(0x2000 + tmp) | (Read(0x2000 + ((tmp+1)&0xFF))<<8);
              sprintf(stringo+strlen(stringo),"  @ $%04X",tmp);
-             if(optable[x].type&1)
+             if(optable[op_index].type&1)
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
              break;
       case 10:
 	     tmp = Read(0x2000 + arg) | (Read(0x2000 + ((arg+1)&0xFF))<<8);
              tmp=(tmp+GetY())&0xFFFF;
              sprintf(stringo+strlen(stringo),"  @ $%04X",tmp);
-             if(optable[x].type&1)
+             if(optable[op_index].type&1)
               sprintf(stringo+strlen(stringo)," = $%02X",Read(tmp));
              break;
 
      }
     }
-
-
    }
    else
    {
-    strcat(stringo,"                  ");
-    strcat(stringo,optable[x].name);
+    strcat(stringo,optable[op_index].name);
    }
+
+   for(int x = strlen(stringo); x < 30; x++)
+   {
+    stringo[x] = ' ';
+    stringo[x + 1] = 0;
+   }
+   sprintf(stringo+strlen(stringo),";%02X ", buf);
+
+   for(int x = 0; x < pow; x++)
+    sprintf(stringo + strlen(stringo), (pow > 4) ? "%02X" : "%02X ", (unsigned int)((arg_orig >> x * 8) & 0xFF));
   }
   else
-   sprintf(stringo+strlen(stringo),"                  .db $%02X",buf);
+   sprintf(stringo+strlen(stringo),".db $%02X                       ;%02X", buf, buf);
 
- return(std::string(stringo));
 }
