@@ -17,10 +17,10 @@
 
 #include "main.h"
 #include <trio/trio.h>
-#include <stdio.h>
 #include <time.h>
 #include "gfxdebugger.h"
 #include "memdebugger.h"
+#include "logdebugger.h"
 #include "prompt.h"
 
 static FILE *TraceLog = NULL;
@@ -31,12 +31,14 @@ static bool NeedInit = 1;
 static bool NeedHooksInstalled = TRUE;
 static bool WatchLogical = 1; // Watch logical memory addresses, not physical
 static bool IsActive = 0;
+static bool CompactMode = FALSE;
 
 static unsigned int WhichMode = 0; // 0 = normal, 1 = gfx, 2 = memory
 
 static std::string ReadBreakpoints, IOReadBreakpoints, AuxReadBreakpoints;
 static std::string WriteBreakpoints, IOWriteBreakpoints, AuxWriteBreakpoints;
 static std::string OpBreakpoints;
+static int DebuggerOpacity;
 
 static void MemPoke(uint32 A, uint32 V, uint32 Size, bool hl, bool logical)
 {
@@ -44,29 +46,29 @@ static void MemPoke(uint32 A, uint32 V, uint32 Size, bool hl, bool logical)
 
  if(logical)
  {
-  for(unsigned int x = 0; x < CurGame->Debugger->AddressSpaces.size(); x++)
-   if(!strcasecmp(CurGame->Debugger->AddressSpaces[x].name, "logical"))
+  for(unsigned int x = 0; x < CurGame->Debugger->AddressSpaces->size(); x++)
+   if(!strcasecmp((*CurGame->Debugger->AddressSpaces)[x].name, "logical"))
    {
-    found = &CurGame->Debugger->AddressSpaces[x]; 
+    found = &(*CurGame->Debugger->AddressSpaces)[x];
     break;
    }
  }
  else
  {
-  for(unsigned int x = 0; x < CurGame->Debugger->AddressSpaces.size(); x++)
-   if(!strcasecmp(CurGame->Debugger->AddressSpaces[x].name, "physical"))
+  for(unsigned int x = 0; x < CurGame->Debugger->AddressSpaces->size(); x++)
+   if(!strcasecmp((*CurGame->Debugger->AddressSpaces)[x].name, "physical"))
    {
-    found = &CurGame->Debugger->AddressSpaces[x];
+    found = &(*CurGame->Debugger->AddressSpaces)[x];
     break;
    }
  }
 
  if(!found)
  {
-  for(unsigned int x = 0; x < CurGame->Debugger->AddressSpaces.size(); x++)
-   if(!strcasecmp(CurGame->Debugger->AddressSpaces[x].name, "cpu"))
+  for(unsigned int x = 0; x < CurGame->Debugger->AddressSpaces->size(); x++)
+   if(!strcasecmp((*CurGame->Debugger->AddressSpaces)[x].name, "cpu"))
    {
-    found = &CurGame->Debugger->AddressSpaces[x];
+    found = &(*CurGame->Debugger->AddressSpaces)[x];
     break;
    }
  }
@@ -150,6 +152,8 @@ static void ParseBreakpoints(const std::string &Breakpoints, int type)
     }
     else
      if(trio_sscanf(zestring + last_x, "%08x%*[-]%08x", &A1, &A2) < 2) continue;
+
+    two_parter = 0;
    }
    else
    {
@@ -184,8 +188,9 @@ static uint32 RegsWhichFont[5];
 static uint32 RegsTotalWidth;
 
 #define MK_COLOR_A(r,g,b,a) ( ((a)<<surface->format->Ashift) | ((r)<<surface->format->Rshift) | ((g) << surface->format->Gshift) | ((b) << surface->format->Bshift))
-//#define DIS_ENTRIES	28
-#define DIS_ENTRIES 58
+static int DIS_ENTRIES = 58;
+static int DisFont = MDFN_FONT_5x7;
+static int DisFontHeight = 7;
 
 static std::vector<uint32> PCBreakPoints;
 static void RedoPCBreakPoints(void)
@@ -290,6 +295,56 @@ static void DrawRegs(RegGroupType *rg, SDL_Surface *surface, uint32 *pixels, int
   }
 }
 
+static void DrawZP(SDL_Surface *surface, uint32 *pixels)
+{
+ uint32 addr = CurGame->Debugger->ZPAddr;
+
+ for(int y = -1; y < 17; y++)
+  for(int x = -1; x < 17; x++)
+  {
+   uint8 zebyte = CurGame->Debugger->MemPeek(addr, 1, 1, TRUE);
+   char tbuf[32];
+   int r, g, b;
+   bool NeedInc = FALSE;
+
+   if((y == -1 || y == 16) && (x == -1 || x == 16)) continue;
+
+   if(y == -1 || y == 16)
+   {
+    r = 0x00;
+    g = 0xE0;
+    b = 0x00;
+    snprintf(tbuf, 32, "x%1X", x);
+   }
+   else if(x == -1 || x == 16)
+   {
+    r = 0x00;
+    g = 0xE0;
+    b = 0x00;
+    snprintf(tbuf, 32, "%1Xx", y);
+   }
+   else
+   {
+    snprintf(tbuf, 32, "%02X", zebyte);
+
+    r = 0x00;
+    g = 0x00;
+    b = 0x00;
+
+    if(x & 4) { r += 0xeF; }
+    if(y & 4) { b += 0xFF; g += 0x80; }
+    if(!(x & 4) && !(y & 4))
+     r = g = b = 0xE0;
+    NeedInc = TRUE;
+   }
+
+   DrawTextTrans(pixels + (x + 1) * 13 + (y + 1) * 10 * (surface->pitch >> 2), surface->pitch, 1024, (UTF8 *)tbuf, MK_COLOR_A(r, g, b, 0xFF), FALSE, MDFN_FONT_5x7);
+   if(NeedInc)
+    addr++;
+  }
+
+}
+
 typedef enum
 {
  None = 0,
@@ -476,8 +531,8 @@ class DebuggerPrompt : public HappyPrompt
 
                    trio_sscanf(tmp_c_str, "%08X", &RegValue);
                    LockGameMutex(1);
-		   if(CurGame->Debugger->RegGroups[(InRegs - 1)]->SetRegister)
-                    CurGame->Debugger->RegGroups[(InRegs - 1)]->SetRegister(PromptText, RegValue);
+		   if((*CurGame->Debugger->RegGroups)[(InRegs - 1)]->SetRegister)
+                    (*CurGame->Debugger->RegGroups)[(InRegs - 1)]->SetRegister(PromptText, RegValue);
 		   else
 		    puts("Null SetRegister!");
 
@@ -518,10 +573,16 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
  }
  else if(WhichMode == 2)
  {
-  SDL_FillRect(surface, rect, MK_COLOR_A(0, 0, 0, 0xc0));
+  SDL_FillRect(surface, rect, MK_COLOR_A(0, 0, 0, DebuggerOpacity));
   MemDebugger_Draw(surface, rect, screen_rect);
   return;  
  }
+ else if(WhichMode == 3)
+ {
+  SDL_FillRect(surface, rect, MK_COLOR_A(0, 0, 0, DebuggerOpacity));
+  LogDebugger_Draw(surface, rect, screen_rect);
+  return;
+ }
  uint32 * pixels = (uint32 *)surface->pixels;
  uint32 pitch32 = surface->pitch >> 2;
 
@@ -531,7 +592,7 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
   for(unsigned int x = 0; x < rect->w; x++)
   {
    //printf("%d %d %d\n", y, x, pixels);
-   row[x] = MK_COLOR_A(0, 0, 0, 0xc0);
+   row[x] = MK_COLOR_A(0, 0, 0, DebuggerOpacity);
    //row[x] = MK_COLOR_A(0x00, 0x00, 0x00, 0x7F);
   }
  }
@@ -544,6 +605,7 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
  // that we have enough disassembled datums for displayaling and
  // worshipping cactus mules.
 
+ 
  int PreBytes = CurGame->Debugger->MaxInstructionSize * (DIS_ENTRIES / 2) * 2;
  int DisBytes = CurGame->Debugger->MaxInstructionSize * (DIS_ENTRIES / 2) * 3;
 
@@ -566,7 +628,6 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
 
   CurGame->Debugger->Disassemble(A, DisAddr, dis_text_buf); // A is passed by reference to Disassemble()
 
-
   DisBliss.push_back(std::string(dis_text_buf));
 
   if(A > DisAddr && lastA < DisAddr) // Err, oops, resynch to DisAddr if necessary
@@ -587,25 +648,25 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
  {
   int32 dbi = indexcow - (DIS_ENTRIES / 2) + x;
 
-  if(dbi < 0 || dbi >= DisBliss.size())
+  if(dbi < 0 || dbi >= (int32)DisBliss.size())
   {
    puts("Disassembly error!");
    break;
   }
 
   std::string dis_str = DisBliss[dbi];
-  uint32 A = DisBlissA[dbi];
+  uint32 dis_A = DisBlissA[dbi];
 
   char addr_text[64];
   uint32 color = MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF);
   uint32 addr_color = MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF);
 
   if(CurGame->Debugger->LogAddrBits == 32)
-   snprintf(addr_text, 256, " %08X: ", A);
+   snprintf(addr_text, 256, " %08X: ", dis_A);
   else
-   snprintf(addr_text, 256, " %04X: ", A);
+   snprintf(addr_text, 256, " %04X: ", dis_A);
   
-  if(A == DisAddr)
+  if(dis_A == DisAddr)
   {
    addr_text[0] = '>';
    if(!InRegs)
@@ -613,20 +674,22 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
 
    if(NeedPCBPToggle)
    {
-    TogglePCBreakPoint(A);
+    TogglePCBreakPoint(dis_A);
     NeedPCBPToggle = 0;
    }
   }
-  if(IsPCBreakPoint(A))
+  if(IsPCBreakPoint(dis_A))
    addr_text[0] = addr_text[0] == '>' ? '#' : '*';
 
-  DrawTextTrans(pixels + x * 7 * pitch32, surface->pitch, rect->w, (UTF8*)addr_text, addr_color, 0, 1);
-  DrawTextTrans(pixels + x * 7 * pitch32 + strlen(addr_text) * 5, surface->pitch, rect->w - strlen(addr_text) * 5, (UTF8*)dis_str.c_str(), color, 0, 1);
+  int addrpixlen;
+
+  addrpixlen = DrawTextTrans(pixels + x * DisFontHeight * pitch32, surface->pitch, rect->w, (UTF8*)addr_text, addr_color, 0, DisFont);
+  DrawTextTrans(pixels + x * DisFontHeight * pitch32 + addrpixlen, surface->pitch, rect->w - strlen(addr_text) * 5, (UTF8*)dis_str.c_str(), color, 0, DisFont);
  }
 
  if(NeedDisAddrChange)
  {
-  if((indexcow + NeedDisAddrChange) >= DisBlissA.size())
+  if((indexcow + NeedDisAddrChange) >= (int32)DisBlissA.size())
   {
    puts("Error, gack!");
   }
@@ -640,41 +703,60 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
  CurRegLongName = "";
  CurRegDetails = "";
 
- for(unsigned int rp = 0; rp < CurGame->Debugger->RegGroups.size(); rp++)
-  DrawRegs(CurGame->Debugger->RegGroups[rp], surface, pixels + rect->w - RegsTotalWidth + RegsColsPixOffset[rp], (InRegs == rp + 1) ? (int)RegsPos : -1, RegsWhichFont[rp]); // 175
+ for(unsigned int rp = 0; rp < CurGame->Debugger->RegGroups->size(); rp++)
+  DrawRegs((*CurGame->Debugger->RegGroups)[rp], surface, pixels + rect->w - RegsTotalWidth + RegsColsPixOffset[rp], (InRegs == rp + 1) ? (int)RegsPos : -1, RegsWhichFont[rp]); // 175
+
+ if(CurGame->Debugger->ZPAddr != (uint32)~0UL)
+  DrawZP(surface, pixels + 324 + 224 * pitch32);
+
+ int moo = 8;
+
+ if(CompactMode)
+  moo = 16;
 
  if(InRegs)
  {
-  DrawTextTrans(pixels + (rect->h - 10 * 7) * pitch32, surface->pitch, surface->w, (UTF8*)CurRegLongName.c_str(), MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF), TRUE, 1);
-  DrawTextTrans(pixels + (rect->h - 9 * 7) * pitch32, surface->pitch, surface->w, (UTF8*)CurRegDetails.c_str(), MK_COLOR_A(0x60, 0xb0, 0xFF, 0xFF), TRUE, 1);
+  DrawTextTrans(pixels + (rect->h - (moo + 2) * 7) * pitch32, surface->pitch, surface->w, (UTF8*)CurRegLongName.c_str(), MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF), TRUE, 1);
+  DrawTextTrans(pixels + (rect->h - (moo + 1) * 7) * pitch32, surface->pitch, surface->w, (UTF8*)CurRegDetails.c_str(), MK_COLOR_A(0x60, 0xb0, 0xFF, 0xFF), TRUE, 1);
  }
  else if(CurGame->Debugger->GetBranchTrace)
  {
   std::vector<std::string> btrace = CurGame->Debugger->GetBranchTrace();
-  uint32 *btpixels = pixels + (rect->h - 10 * 7) * pitch32;
+  uint32 *btpixels = pixels + (rect->h - (moo + 2) * 7) * pitch32;
   char strbuf[256];
   int strbuf_len = 0;
+  int btrace_rows = 2;
+  int btrace_index = 0;
 
-  for(unsigned int y = 0; y < btrace.size() / 2; y++)
+  for(int bt_row = 0; bt_row < btrace_rows; bt_row++)
   {
-   strbuf_len += sprintf(strbuf + strbuf_len, "%s→", btrace[y].c_str());
-  }
-  DrawTextTrans(btpixels, surface->pitch, rect->w, (UTF8*)strbuf, MK_COLOR_A(0x60, 0xb0, 0xfF, 0xFF), TRUE, 1);
+   strbuf_len = 0;
+   for(unsigned int y = 0; y < btrace.size() / btrace_rows; y++)
+   {
+    strbuf_len += sprintf(strbuf + strbuf_len, "%s→", btrace[btrace_index].c_str());
+    btrace_index++;
+   }
 
-  strbuf_len = 0;
-  for(unsigned int y = btrace.size() / 2; y < btrace.size(); y++)
-  {
-   strbuf_len += sprintf(strbuf + strbuf_len, "%s→", btrace[y].c_str());
+   if(bt_row == (btrace_rows - 1))
+    strbuf[strbuf_len - 1] = 0; // Get rid of the trailing →
+
+   DrawTextTrans(btpixels + bt_row * 7 * pitch32, surface->pitch, rect->w, (UTF8*)strbuf, MK_COLOR_A(0x60, 0xb0, 0xfF, 0xFF), TRUE, MDFN_FONT_5x7);
   }
-  strbuf[strbuf_len - 1] = 0; // Get rid of the trailing →
-  DrawTextTrans(btpixels + 7 * pitch32, surface->pitch, rect->w, (UTF8*)strbuf, MK_COLOR_A(0x60, 0xb0, 0xfF, 0xFF), TRUE, 1);
  }
 
  // Draw memory watch section
  {
-  uint32 *watchpixels = pixels + (rect->h - 8 * 7) * pitch32;
- 
-  for(int y = 0; y < 8; y++)
+  uint32 *watchpixels = pixels + (rect->h - moo * 7) * pitch32;
+  int mw_rows = 8;
+  int bytes_per_row = 32;
+
+  if(CompactMode) 
+   mw_rows = 16;
+
+  if(CompactMode)
+   bytes_per_row = 16;
+
+  for(int y = 0; y < mw_rows; y++)
   {
    uint32 *row = watchpixels + y * pitch32 * 7;
    char tbuf[256];
@@ -683,7 +765,7 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
    uint32 ewa_bits;
    uint32 ewa_mask;
 
-   asciistr[16] = 0;  
+   asciistr[32] = 0;  
 
    if(WatchLogical)
    {
@@ -699,18 +781,18 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
    ewa_mask = ((uint64)1 << ewa_bits) - 1;
 
    if(ewa_bits <= 16)
-    snprintf(tbuf, 256, "%04X: ", (ewa + y * 32) & ewa_mask);
+    snprintf(tbuf, 256, "%04X: ", (ewa + y * bytes_per_row) & ewa_mask);
    else if(ewa_bits <= 20)
-    snprintf(tbuf, 256, "%05X: ", (ewa + y * 32) & ewa_mask);
+    snprintf(tbuf, 256, "%05X: ", (ewa + y * bytes_per_row) & ewa_mask);
    else if(ewa_bits <= 24)
-    snprintf(tbuf, 256, "%06X: ", (ewa + y * 32) & ewa_mask);
+    snprintf(tbuf, 256, "%06X: ", (ewa + y * bytes_per_row) & ewa_mask);
    else
-    snprintf(tbuf, 256, "%08X: ", (ewa + y * 32) & ewa_mask);
+    snprintf(tbuf, 256, "%08X: ", (ewa + y * bytes_per_row) & ewa_mask);
 
-   row += DrawTextTrans(row, surface->pitch, rect->w, (UTF8 *)tbuf, MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF), 0, 1);
-   for(int x = 0; x < 32; x++)
+   row += DrawTextTrans(row, surface->pitch, rect->w, (UTF8 *)tbuf, MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF), FALSE, MDFN_FONT_5x7);
+   for(int x = 0; x < bytes_per_row; x++)
    {
-    uint8 zebyte = CurGame->Debugger->MemPeek((ewa + y * 32 + x) & ewa_mask, 1, 1, WatchLogical);
+    uint8 zebyte = CurGame->Debugger->MemPeek((ewa + y * bytes_per_row + x) & ewa_mask, 1, 1, WatchLogical);
     uint32 bcolor = MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF);
 
     if(x & 1)
@@ -726,7 +808,7 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
     snprintf(tbuf, 256, "%02X", zebyte);
     row += DrawTextTrans(row, surface->pitch, rect->w, (UTF8*)tbuf, bcolor, 0, 1) + 2;
    }
-   DrawTextTrans(row + 5, surface->pitch, rect->w, (UTF8 *)asciistr, MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+   DrawTextTrans(row + 5, surface->pitch, rect->w, (UTF8 *)asciistr, MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, MDFN_FONT_5x7);
   }
  }  
 
@@ -830,9 +912,10 @@ bool Debugger_IsActive(unsigned int *w, unsigned int *h)
  switch(WhichMode)
  {
   default:
-  case 0: if(w) *w = 640; if(h) *h = 480; break;
+  case 0: if(w) *w = CompactMode ? 512 : 640; if(h) *h = CompactMode ? 448 : 480; break;
   case 1: if(w) *w = 384; if(h) *h = 320; break;
   case 2: if(w) *w = 320; if(h) *h = 240; break;
+  case 3: if(w) *w = 512; if(h) *h = 448; break;
  }
 
  return(IsActive);
@@ -850,9 +933,38 @@ bool Debugger_Toggle(void)
   {
    if(NeedInit)
    {
+    std::string des_disfont = MDFN_GetSettingS(std::string(std::string(CurGame->shortname) + "." + std::string("debugger.disfontsize")).c_str());
+    DebuggerOpacity = 0xC0;
+
+    //if(!strcasecmp(CurGame->shortname, "nes") || !strcasecmp(CurGame->shortname, "pce"))
+    if(des_disfont == "xsmall")
+    {
+     DisFont = MDFN_FONT_4x5;
+     DisFontHeight = 6;
+    }
+    else if(des_disfont == "medium")
+    {
+     DisFont = MDFN_FONT_6x13_12x13;
+     DisFontHeight = 12;
+    }
+    else if(des_disfont == "large")
+    {
+     DisFont = MDFN_FONT_9x18_18x18;
+     DisFontHeight = 17;
+    }
+    else // small
+    {
+     DisFont = MDFN_FONT_5x7;
+     DisFontHeight = 7;
+    }
+
+    DIS_ENTRIES = 406 / DisFontHeight;
+
+    //DIS_ENTRIES = CompactMode ? 46 : 58;
+
     NeedInit = FALSE;
     WatchAddr = CurGame->Debugger->DefaultWatchAddr;
-    DisAddr = CurGame->Debugger->RegGroups[0]->GetRegister("PC", NULL);
+    DisAddr = (*CurGame->Debugger->RegGroups)[0]->GetRegister("PC", NULL);
 
     RegsCols = 0;
     RegsTotalWidth = 0;
@@ -860,16 +972,16 @@ bool Debugger_Toggle(void)
     memset(RegsColsCounts, 0, sizeof(RegsColsCounts));
 
     int pw_offset = 0;
-    for(unsigned int r = 0; r < CurGame->Debugger->RegGroups.size(); r++)
+    for(unsigned int r = 0; r < CurGame->Debugger->RegGroups->size(); r++)
     {
      uint32 pw = 0;
      int x;
 
-     for(x = 0; CurGame->Debugger->RegGroups[r]->Regs[x].bsize; x++)
+     for(x = 0; (*CurGame->Debugger->RegGroups)[r]->Regs[x].bsize; x++)
      {
-      if(CurGame->Debugger->RegGroups[r]->Regs[x].bsize != 0xFFFF)
+      if((*CurGame->Debugger->RegGroups)[r]->Regs[x].bsize != 0xFFFF)
       {
-       uint32 tmp_pw = (strlen(CurGame->Debugger->RegGroups[r]->Regs[x].name.c_str()) + CurGame->Debugger->RegGroups[r]->Regs[x].bsize * 2 + 2 + 2);
+       uint32 tmp_pw = (strlen((*CurGame->Debugger->RegGroups)[r]->Regs[x].name.c_str()) + (*CurGame->Debugger->RegGroups)[r]->Regs[x].bsize * 2 + 2 + 2);
 
        if(tmp_pw > pw)
         pw = tmp_pw;
@@ -878,10 +990,10 @@ bool Debugger_Toggle(void)
       RegsColsCounts[r]++;
      }
 
-     if(r == (CurGame->Debugger->RegGroups.size() - 1))
+     if(r == (CurGame->Debugger->RegGroups->size() - 1))
       pw -= 2;
 
-     if(x > 58)
+     if(x * 7 > (DIS_ENTRIES * DisFontHeight))
      {
       pw *= 4;
       RegsWhichFont[r] = MDFN_FONT_4x5;
@@ -916,12 +1028,14 @@ bool Debugger_Toggle(void)
      GfxDebugger_SetActive(TRUE);
     else if(WhichMode == 2)
      MemDebugger_SetActive(TRUE);
+    else if(WhichMode == 3)
+     LogDebugger_SetActive(TRUE);
    }
   }
   else // Disabled, yahyahyah
   {
    // Only uninstall our hooks if we don't have any active breakpoints, and trace log is off.
-   if(!ReadBreakpoints.size() && !WriteBreakpoints.size() && !IOReadBreakpoints.size() && 
+   if(!PCBreakPoints.size() && !ReadBreakpoints.size() && !WriteBreakpoints.size() && !IOReadBreakpoints.size() && 
 	!IOWriteBreakpoints.size() && !AuxReadBreakpoints.size() && !AuxWriteBreakpoints.size()
 	&& !TraceLog)
    {
@@ -930,39 +1044,62 @@ bool Debugger_Toggle(void)
     CurGame->Debugger->SetBPCallback(FALSE);
     GfxDebugger_SetActive(FALSE);
     MemDebugger_SetActive(FALSE);
+    LogDebugger_SetActive(FALSE);
    }
   }
+  SDL_MDFN_ShowCursor(IsActive);
  }
  return(IsActive);
 }
 
+void Debugger_ModOpacity(int deltalove)
+{
+ DebuggerOpacity += deltalove;
+ if(DebuggerOpacity < 0) DebuggerOpacity = 0;
+ if(DebuggerOpacity > 0xFF) DebuggerOpacity = 0xFF;
+}
 
 // Called from the main thread
 void Debugger_Event(const SDL_Event *event)
 {
-  if(event->type == SDL_KEYDOWN && event->key.keysym.mod & KMOD_ALT)
+  if(event->type == SDL_KEYDOWN)
   {
-   switch(event->key.keysym.sym)
+   if(event->key.keysym.mod & KMOD_ALT)
    {
-    case SDLK_1: WhichMode = 0; 
-		 GfxDebugger_SetActive(FALSE);
-		 MemDebugger_SetActive(FALSE);
-		 break;
-    case SDLK_2: WhichMode = 1;
-		 GfxDebugger_SetActive(TRUE); 
-		 MemDebugger_SetActive(FALSE);
-		 break;
-    case SDLK_3: if(CurGame->Debugger->AddressSpaces.size())
-		 {
-		  WhichMode = 2;
-		  GfxDebugger_SetActive(FALSE);
-		  MemDebugger_SetActive(TRUE);
-		 }
-		 break;
+    switch(event->key.keysym.sym)
+    {
+     case SDLK_1: WhichMode = 0; 
+ 		  GfxDebugger_SetActive(FALSE);
+ 		  MemDebugger_SetActive(FALSE);
+		  LogDebugger_SetActive(FALSE);
+		  break;
+     case SDLK_2: WhichMode = 1;
+		  GfxDebugger_SetActive(TRUE); 
+		  MemDebugger_SetActive(FALSE);
+		  LogDebugger_SetActive(FALSE);
+		  break;
+     case SDLK_3: if(CurGame->Debugger->AddressSpaces->size())
+		  {
+		   WhichMode = 2;
+		   GfxDebugger_SetActive(FALSE);
+		   MemDebugger_SetActive(TRUE);
+		   LogDebugger_SetActive(FALSE);
+		  }
+		  break;
+     case SDLK_4: if(CurGame->Debugger->SetLogFunc)
+		  {
+		   WhichMode = 3;
+		   GfxDebugger_SetActive(FALSE);
+		   MemDebugger_SetActive(FALSE);
+		   LogDebugger_SetActive(TRUE);
+		  }
+		  break;
 
-    default: break;
+     default: break;
+    }
    }
   }
+
   if(WhichMode == 1)
   {
    GfxDebugger_Event(event);
@@ -973,6 +1110,12 @@ void Debugger_Event(const SDL_Event *event)
    MemDebugger_Event(event);
    return;
   }
+  else if(WhichMode == 3)
+  {
+   LogDebugger_Event(event);
+   return;
+  }
+
   switch(event->type)
   {
    case SDL_KEYDOWN:
@@ -993,6 +1136,11 @@ void Debugger_Event(const SDL_Event *event)
         else switch(event->key.keysym.sym)
         {
 	 default: break;
+	 case SDLK_MINUS: Debugger_ModOpacity(-8);
+        	          break;
+	 case SDLK_EQUALS: Debugger_ModOpacity(8);
+	                   break;
+
 	 case SDLK_HOME:
 		if(event->key.keysym.mod & KMOD_SHIFT)
 		{
@@ -1213,15 +1361,15 @@ void Debugger_Event(const SDL_Event *event)
 		  {
 		   if(InRegs)
 		   {
-		    if(CurGame->Debugger->RegGroups[InRegs - 1]->Regs[RegsPos].bsize == 0xFFFF)
+		    if((*CurGame->Debugger->RegGroups)[InRegs - 1]->Regs[RegsPos].bsize == 0xFFFF)
 		     break;
 
 		    InPrompt = (PromptType)(EditRegs);
-		    ptext = CurGame->Debugger->RegGroups[InRegs - 1]->Regs[RegsPos].name;
-		    int len = CurGame->Debugger->RegGroups[InRegs - 1]->Regs[RegsPos].bsize;
+		    ptext = (*CurGame->Debugger->RegGroups)[InRegs - 1]->Regs[RegsPos].name;
+		    int len = (*CurGame->Debugger->RegGroups)[InRegs - 1]->Regs[RegsPos].bsize;
 
 		    LockGameMutex(1);
-		    uint32 RegValue = CurGame->Debugger->RegGroups[InRegs - 1]->GetRegister(ptext, NULL);
+		    uint32 RegValue = (*CurGame->Debugger->RegGroups)[InRegs - 1]->GetRegister(ptext, NULL);
 		    LockGameMutex(0);
 
 		    if(len == 1)
