@@ -23,8 +23,6 @@
 #include <string.h>
 #include <strings.h>
 #include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <trio/trio.h>
 #include <locale.h>
 
@@ -50,6 +48,7 @@
 #include "video-state.h"
 #include "remote.h"
 
+static bool RemoteOn = FALSE;
 bool pending_save_state, pending_snapshot, pending_save_movie;
 static volatile Uint32 MainThreadID = 0;
 static bool ffnosound;
@@ -125,6 +124,20 @@ static MDFNSetting DriverSettings[] =
   { "gba.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
   #endif
 
+  { "gg.xres", CSD_xres, MDFNST_UINT, "640" },
+  { "gg.yres", CSD_yres, MDFNST_UINT, "480" },
+  { "gg.xscale", CSD_xscale, MDFNST_FLOAT, "3" },
+  { "gg.yscale", CSD_yscale, MDFNST_FLOAT, "3" },
+  { "gg.xscalefs", CSD_xscalefs, MDFNST_FLOAT, "2" },
+  { "gg.yscalefs", CSD_yscalefs, MDFNST_FLOAT, "2" },
+  { "gg.scanlines", CSD_scanlines, MDFNST_UINT, "0" },
+  { "gg.stretch", CSD_stretch, MDFNST_BOOL, "0" },
+  { "gg.videoip", CSD_videoip, MDFNST_BOOL, "0" },
+  { "gg.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "gg.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
+
   { "lynx.xres", CSD_xres, MDFNST_UINT, "800" },
   { "lynx.yres", CSD_yres, MDFNST_UINT, "600" },
   { "lynx.xscale", CSD_xscale, MDFNST_FLOAT, "4" },
@@ -195,6 +208,20 @@ static MDFNSetting DriverSettings[] =
   { "player.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
   #endif
 
+  { "sms.xres", CSD_xres, MDFNST_UINT, "640" },
+  { "sms.yres", CSD_yres, MDFNST_UINT, "480" },
+  { "sms.xscale", CSD_xscale, MDFNST_FLOAT, "3" },
+  { "sms.yscale", CSD_yscale, MDFNST_FLOAT, "3" },
+  { "sms.xscalefs", CSD_xscalefs, MDFNST_FLOAT, "2" },
+  { "sms.yscalefs", CSD_yscalefs, MDFNST_FLOAT, "2" },
+  { "sms.scanlines", CSD_scanlines, MDFNST_UINT, "0" },
+  { "sms.stretch", CSD_stretch, MDFNST_BOOL, "0" },
+  { "sms.videoip", CSD_videoip, MDFNST_BOOL, "0" },
+  { "sms.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "sms.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
+
   { "wswan.xres", CSD_xres, MDFNST_UINT, "640" },
   { "wswan.yres", CSD_yres, MDFNST_UINT, "480" },
   { "wswan.xscale", CSD_xscale, MDFNST_FLOAT, "3" },
@@ -210,7 +237,7 @@ static MDFNSetting DriverSettings[] =
   #endif
 
   { "fs", gettext_noop("Enable fullscreen mode."), MDFNST_BOOL, "0", },
-  { "vdriver", gettext_noop("Select video driver.  0 = OpenGL, 1 = SDL surface blits"), MDFNST_UINT, "0", "0", "1" },
+  { "vdriver", gettext_noop("Select video driver, \"opengl\" or \"sdl\"."), MDFNST_STRING, "opengl", NULL, NULL, MDFND_ValidateVideoSetting },
   { "glvsync", gettext_noop("Attempt to synchronize OpenGL page flips to vertical retrace period."), MDFNST_BOOL, "1" },
 
   { "ffspeed", gettext_noop("Fast-forwarding speed multiplier."), MDFNST_UINT, "4", "1", "15" },
@@ -234,6 +261,19 @@ static MDFNSetting DriverSettings[] =
    ,"1", "1000" },
   { "soundrate", gettext_noop("Specifies the sound playback rate, in frames per second(\"Hz\")."), MDFNST_UINT, "48000", "8192", "48000"},
   { "helpenabled", gettext_noop("Enable the help screen."), MDFNST_BOOL, "1" },
+
+  #ifdef WANT_DEBUGGER
+  { "nes.debugger.memcharset", gettext_noop("Character set for the debugger's memory editor."), MDFNST_STRING, "UTF-8" },
+  { "pce.debugger.memcharset", gettext_noop("Character set for the debugger's memory editor."), MDFNST_STRING, "shift_jis" },
+  { "pcfx.debugger.memcharset", gettext_noop("Character set for the debugger's memory editor."), MDFNST_STRING, "shift_jis" },
+  { "wswan.debugger.memcharset", gettext_noop("Character set for the debugger's memory editor."), MDFNST_STRING, "UTF-8" },
+
+  { "nes.debugger.disfontsize", gettext_noop("Disassembly font size(xsmall, small, medium, large)."), MDFNST_STRING, "small" },
+  { "pce.debugger.disfontsize", gettext_noop("Disassembly font size(xsmall, small, medium, large)."), MDFNST_STRING, "small" },
+  { "pcfx.debugger.disfontsize", gettext_noop("Disassembly font size(xsmall, small, medium, large)."), MDFNST_STRING, "small" },
+  { "wswan.debugger.disfontsize", gettext_noop("Disassembly font size(xsmall, small, medium, large)."), MDFNST_STRING, "small" },
+
+  #endif
 };
 
 
@@ -276,7 +316,10 @@ void MDFND_PrintError(const char *s)
  }
  else
  {
-  puts(s);
+  if(RemoteOn)
+   Remote_SendErrorMessage(s);
+  else
+   puts(s);
  }
 }
 
@@ -292,12 +335,17 @@ void MDFND_Message(const char *s)
   SDL_PushEvent(&evt);
  }
  else
-  fputs(s,stdout);
+ {
+  if(RemoteOn)
+   Remote_SendStatusMessage(s);
+  else
+   fputs(s,stdout);
+ }
 }
 
 static void CreateDirs(void)
 {
- char *subs[6]={"mcs","mcm","snaps","gameinfo","sav","cheats"};
+ const char *subs[6]={"mcs","mcm","snaps","gameinfo","sav","cheats"};
  char *tdir;
  int x;
 
@@ -421,7 +469,7 @@ static void DeleteInternalArgs(void)
 
  do
  {
-  free(argptr->name);
+  free((void*)argptr->name);
   argptr++;
  } while(argptr->name || argptr->var || argptr->subs);
  free(MDFN_Internal_Args);
@@ -456,9 +504,11 @@ static char * loadcd = NULL;
 static int DoArgs(int argc, char *argv[], char **filename)
 {
 	int ShowCLHelp = 0;
+	int DoSetRemote = 0;
 
         ARGPSTRUCT MDFNArgs[]={
 	 {"help", gettext_noop("Show help!"), &ShowCLHelp, 0, 0 },
+	 {"remote", gettext_noop("Enable remote mode."), &DoSetRemote, 0, 0 },
 	 #ifdef NEED_CDEMU
 	 {"loadcd", gettext_noop("Load and boot a CD for the specified system."), 0, &loadcd, 0x4001},
 	 #endif
@@ -530,12 +580,14 @@ int LoadGame(const char *path)
 	  return 0;
 	}
 	CurGame = tmp;
-	InitOtherInput(tmp);
+	InitGameInput(tmp);
 
         RefreshThrottleFPS(1);
+
         SDL_mutexP(VTMutex);
         NeedVideoChange = -1;
         SDL_mutexV(VTMutex);
+
         if(SDL_ThreadID() != MainThreadID)
           while(NeedVideoChange)
 	  {
@@ -567,18 +619,16 @@ int CloseGame(void)
 	GameThreadRun = 0;
 
 	SDL_WaitThread(GameThread, NULL);
+
 	if(MDFN_GetSettingB("autosave"))
 	 MDFNI_SaveState(NULL, "ncq", NULL, NULL);
 
-	ResyncOtherInput();
-
 	MDFNI_CloseGame();
 
-        KillOtherInput();
-	KillJoysticks();
+        KillGameInput();
 	KillSound();
 
-	CurGame = 0;
+	CurGame = NULL;
 
 	if(soundrecfn)
          MDFNI_EndWaveRecord();
@@ -667,7 +717,19 @@ int GameLoop(void *arg)
 	 int ThisBackBuffer = VTBackBuffer;
 
 	 LockGameMutex(1);
-         MDFNI_Emulate((uint32 *)VTBuffer[VTBackBuffer], (MDFN_Rect *)VTLineWidths[VTBackBuffer], &sound, &ssize, fskip, CurGameSpeed);
+	 {
+	  EmulateSpecStruct espec;
+ 	  memset(&espec, 0, sizeof(EmulateSpecStruct));
+
+	  espec.pixels = (uint32 *)VTBuffer[VTBackBuffer];
+	  espec.LineWidths = (MDFN_Rect *)VTLineWidths[VTBackBuffer];
+	  espec.SoundBuf = &sound;
+	  espec.SoundBufSize = &ssize;
+	  espec.skip = fskip;
+	  espec.soundmultiplier = CurGameSpeed;
+	  espec.NeedRewind = DNeedRewind;
+          MDFNI_Emulate(&espec); //(uint32 *)VTBuffer[VTBackBuffer], (MDFN_Rect *)VTLineWidths[VTBackBuffer], &sound, &ssize, fskip, CurGameSpeed);
+	 }
 	 LockGameMutex(0);
 	 FPS_IncVirtual();
 	 if(!fskip)
@@ -726,24 +788,35 @@ char *GetBaseDirectory(void)
 
 static volatile int (*EventHook)(const SDL_Event *event) = 0;
 
-static volatile SDL_Event gtevents[16384];
+static const int gtevents_size = 2048; // Must be a power of 2.
+static volatile SDL_Event gtevents[gtevents_size];
 static volatile int gte_read = 0;
 static volatile int gte_write = 0;
 
 /* This function may also be called by the main thread if a game is not loaded. */
 static void GameThread_HandleEvents(void)
 {
+ SDL_Event gtevents_temp[gtevents_size];
+ int numevents = 0;
+
  SDL_mutexP(EVMutex);
  while(gte_read != gte_write)
  {
-  volatile SDL_Event *event = &gtevents[gte_read];
+  memcpy(&gtevents_temp[numevents], (void *)&gtevents[gte_read], sizeof(SDL_Event));
+
+  numevents++;
+  gte_read = (gte_read + 1) & (gtevents_size - 1);
+ }
+ SDL_mutexV(EVMutex);
+
+ for(int i = 0; i < numevents; i++)
+ {
+  SDL_Event *event = &gtevents_temp[i];
 
   if(EventHook)
-   EventHook((const SDL_Event *)event);
+   EventHook(event);
 
-  NetplayEventHook_GT((const SDL_Event *)event);
-
-  gte_read = (gte_read + 1) & 16383;
+  NetplayEventHook_GT(event);
  }
  SDL_mutexV(EVMutex);
 }
@@ -781,6 +854,9 @@ static bool krepeat = 0;
 void PumpWrap(void)
 {
  SDL_Event event;
+ SDL_Event gtevents_temp[gtevents_size];
+ int numevents = 0;
+
  bool NITI;
 
  NITI = Netplay_IsTextInput();
@@ -797,7 +873,7 @@ void PumpWrap(void)
    SDL_EnableKeyRepeat(0, 0);
   krepeat = 0;
  }
- SDL_mutexP(EVMutex);
+
  while(SDL_PollEvent(&event))
  {
   if(Debugger_IsActive())
@@ -830,6 +906,8 @@ void PumpWrap(void)
   /* Handle the event, and THEN hand it over to the GUI. Order is important due to global variable mayhem(CEVT_TOGGLEFS. */
   switch(event.type)
   {
+   case SDL_ACTIVEEVENT: break;
+   case SDL_SYSWMEVENT: break;
    case SDL_VIDEORESIZE: VideoResize(event.resize.w, event.resize.h); break;
    case SDL_VIDEOEXPOSE: break;
    case SDL_QUIT: NeedExitNow = 1;break;
@@ -856,13 +934,31 @@ void PumpWrap(void)
 		 case CEVT_PRINTMESSAGE: MDFND_Message((char *)event.user.data1); free(event.user.data1); break;
 		 case CEVT_SHOWCURSOR: SDL_ShowCursor(*(int *)event.user.data1); free(event.user.data1); break;
 	  	 case CEVT_DISP_MESSAGE: VideoShowMessage((UTF8*)event.user.data1); break;
-		 default: memcpy((void *)&gtevents[gte_write], &event, sizeof(SDL_Event)); gte_write = (gte_write + 1) & 16383; break;
+		 default: 
+			if(numevents < gtevents_size)
+			{
+			 memcpy(&gtevents_temp[numevents], &event, sizeof(SDL_Event));
+			 numevents++;
+			}
+			break;
 		}
 		break;
-   default: memcpy((void *)&gtevents[gte_write], &event, sizeof(SDL_Event)); gte_write = (gte_write + 1) & 16383; break;
+   default: 
+           if(numevents < gtevents_size)
+           {
+            memcpy(&gtevents_temp[numevents], &event, sizeof(SDL_Event));
+            numevents++;
+           }
+	   break;
   }
  }
 
+ SDL_mutexP(EVMutex);
+ for(int i = 0; i < numevents; i++)
+ {
+  memcpy((void *)&gtevents[gte_write], &gtevents_temp[i], sizeof(SDL_Event));
+  gte_write = (gte_write + 1) & (gtevents_size - 1);
+ }
  SDL_mutexV(EVMutex);
 
  if(!CurGame)
@@ -891,6 +987,8 @@ void SetJoyReadMode(int mode)	// 0 for events, 1 for manual function calling to
 
 bool MT_FromRemote_SoundSync(void)
 {
+ bool ret = TRUE;
+
  GameThreadRun = 0;
  SDL_WaitThread(GameThread, NULL);
 
@@ -898,18 +996,23 @@ bool MT_FromRemote_SoundSync(void)
  sound_active = 0;
 
  if(MDFN_GetSettingB("sound"))
+ {
   sound_active = InitSound(CurGame);
-
+  if(!sound_active)
+   ret = FALSE;
+ }
  GameThreadRun = 1;
  GameThread = SDL_CreateThread(GameLoop, NULL);
+
+ return(ret);
 }
 
 bool MT_FromRemote_VideoSync(void)
 {
-          KillVideo(0);
+          KillVideo();
 
-          memset(VTBuffer[0], 0, 256 * 256 * sizeof(uint32));
-          memset(VTBuffer[1], 0, 256 * 256 * sizeof(uint32));
+          memset(VTBuffer[0], 0, CurGame->pitch * 256);
+          memset(VTBuffer[1], 0, CurGame->pitch * 256);
 
           if(!InitVideo(CurGame))
 	   return(0);
@@ -927,7 +1030,13 @@ void RefreshThrottleFPS(int multiplier)
         CurGameSpeed = multiplier;
 }
 
-extern uint32 uppow2(uint32);
+void PrintSDLVersion(void)
+{
+ const SDL_version *sver = SDL_Linked_Version();
+
+ MDFN_printf(_("Compiled against SDL %u.%u.%u, running with SDL %u.%u.%u\n\n"), SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL, sver->major, sver->minor, sver->patch);
+}
+
 int sdlhaveogl = 0;
 
 int main(int argc, char *argv[])
@@ -952,12 +1061,16 @@ int main(int argc, char *argv[])
 
 	if(SDL_Init(SDL_INIT_VIDEO)) /* SDL_INIT_VIDEO Needed for (joystick config) event processing? */
 	{
-	 printf("Could not initialize SDL: %s.\n", SDL_GetError());
+	 fprintf(stderr, "Could not initialize SDL: %s.\n", SDL_GetError());
 	 MDFNI_Kill();
 	 return(-1);
 	}
 
-	InitSTDIOInterface();
+	if(argc >= 2 && (!strcasecmp(argv[1], "-remote") || !strcasecmp(argv[1], "--remote")))
+         RemoteOn = TRUE;
+
+	if(RemoteOn)
+ 	 InitSTDIOInterface();
 
 	for(unsigned int x = 0; x < sizeof(DriverSettings) / sizeof(MDFNSetting); x++)
 	 NeoDriverSettings.push_back(DriverSettings[x]);
@@ -969,6 +1082,8 @@ int main(int argc, char *argv[])
         if(!(ret=MDFNI_Initialize(DrBaseDirectory, NeoDriverSettings)))
          return(-1);
 
+	PrintSDLVersion();
+
         SDL_EnableUNICODE(1);
 
         #ifdef HAVE_SIGNAL
@@ -982,7 +1097,7 @@ int main(int argc, char *argv[])
 	{
 	 MDFNI_Kill();
 	 DeleteInternalArgs();
-	 KillOtherInputSettings();
+	 KillInputSettings();
 	 return(-1);
 	}
 
@@ -1016,6 +1131,7 @@ int main(int argc, char *argv[])
 	NeedVideoChange = -1;
 
 	InitJoysticks();
+	InitCommandInput();
 
 	NeedExitNow = 0;
 
@@ -1029,18 +1145,19 @@ int main(int argc, char *argv[])
 	 VTLineWidths[0] = (MDFN_Rect *)calloc(256, sizeof(MDFN_Rect));
 	 VTLineWidths[1] = (MDFN_Rect *)calloc(256, sizeof(MDFN_Rect));
          NeedVideoChange = -1;
+         FPS_Init();
+
+         #ifdef WANT_DEBUGGER
+         MemDebugger_Init();
+         #endif
         }
 	else
 	 NeedExitNow = 1;
 
-	FPS_Init();
-	#ifdef WANT_DEBUGGER
-	MemDebugger_Init();
-	#endif
-
 	while(!NeedExitNow)
 	{
-	 CheckForSTDIOMessages();
+	 if(RemoteOn)
+	  CheckForSTDIOMessages();
 
 	 SDL_mutexP(VTMutex);	/* Lock mutex */
 
@@ -1058,10 +1175,10 @@ int main(int argc, char *argv[])
 
 	 if(NeedVideoChange)
 	 {
-	  KillVideo(0);
+	  KillVideo();
 
-	  memset(VTBuffer[0], 0, 256 * 256 * sizeof(uint32));
-	  memset(VTBuffer[1], 0, 256 * 256 * sizeof(uint32));
+	  memset(VTBuffer[0], 0, CurGame->pitch * 256);
+	  memset(VTBuffer[1], 0, CurGame->pitch * 256);
 
 	  if(NeedVideoChange == -1)
 	  {
@@ -1118,11 +1235,18 @@ int main(int argc, char *argv[])
 	SetSignals(SIG_IGN);
 	#endif
 
+	KillCommandInput();
+
         MDFNI_Kill();
+
+	KillJoysticks();
+
+	KillVideo();
+
 	SDL_Quit();
 
 	DeleteInternalArgs();
-	KillOtherInputSettings();
+	KillInputSettings();
 
         return(0);
 }
@@ -1157,6 +1281,7 @@ static int ThrottleCheckFS(void)
  }
 
  if(!MDFNDnetplay)
+ {
   if(((ttime-ltime) >= (1.5*tfreq/desiredfps)))
   {
    //MDFN_DispMessage((UTF8*)"%8d %8d %8d, %8d", ttime, ltime, ttime-ltime, tfreq / desiredfps);
@@ -1175,6 +1300,7 @@ static int ThrottleCheckFS(void)
   }
   else
    ltime+=tfreq/desiredfps;
+ }
 
  return(needskip);
 }
