@@ -16,7 +16,6 @@
  */
 
 #include <string.h>
-#include <stdlib.h>
 
 #include "main.h"
 #include "opengl.h"
@@ -78,28 +77,41 @@ glDeleteObjectARB_Func p_glDeleteObjectARB;
 glDetachObjectARB_Func p_glDetachObjectARB;
 #endif
 
-static uint32 MaxTextureSize;
+static uint32 MaxTextureSize; // Maximum power-of-2 texture width/height(we assume they're the same, and if they're not, this is set to the lower value of the two)
+static bool SupportNPOT; // True if the OpenGL implementation supports non-power-of-2-sized textures
 
-static SDL_Surface *gl_screen;
-static GLuint textures[4]={0, 0, 0, 0}; // emulated fb, scanlines, osd, raw(netplay)
-static GLuint rgb_mask;
+static SDL_Surface *gl_screen = NULL;
+static GLuint textures[4] = {0, 0, 0, 0}; // emulated fb, scanlines, osd, raw(netplay)
+static GLuint rgb_mask = 0; // TODO:  RGB mask texture for LCD RGB triad simulation
 
-static int using_scanlines = 0;
-static int last_w, last_h;
+static bool using_scanlines = 0;
+static unsigned int last_w, last_h;
 static float twitchy;
 
 extern uint32 uppow2(uint32);
-
 static uint32 OSDLastWidth, OSDLastHeight;
 
-#if MDFN_WANT_OPENGL_SHADERS
-static bool UsingShader;
-#endif
+static bool UsingShader = FALSE; // TRUE if we're using a pixel shader.
+static bool UsingIP;	// True if bilinear interpolation is enabled.
+
+static uint32 *DummyBlack; // Black/Zeroed image data for cleaning textures
+static uint32 DummyBlackSize;
 
 void BlitOpenGLRaw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect *dest_rect)
 {
- unsigned int tmpwidth = uppow2(rect->w);
- unsigned int tmpheight = uppow2(rect->h);
+ unsigned int tmpwidth;
+ unsigned int tmpheight;
+
+ if(SupportNPOT)
+ {
+  tmpwidth = rect->w;
+  tmpheight = rect->h;
+ }
+ else
+ {
+  tmpwidth = uppow2(rect->w);
+  tmpheight = uppow2(rect->h);
+ }
 
  if(tmpwidth > MaxTextureSize || tmpheight > MaxTextureSize)
  {
@@ -171,23 +183,93 @@ void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Re
 
  p_glBindTexture(GL_TEXTURE_2D, textures[0]);
 
- int tmpwidth = uppow2(src_rect->w);
- int tmpheight = uppow2(src_rect->h);
-
+ unsigned int tmpwidth;
+ unsigned int tmpheight;
  uint32 *src_pixies = (uint32 *)src_surface->pixels;
 
- if(last_w != tmpwidth || last_h != tmpheight)
- {
-  p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tmpwidth, tmpheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
-  last_w = tmpwidth;
-  last_h = tmpheight;
- }
  src_pixies += left + top * (src_surface->pitch >> 2);
  right -= left;
  left = 0;
  bottom -= top;
  top = 0;
 
+ if(SupportNPOT)
+ {
+  tmpwidth = src_rect->w;
+  tmpheight = src_rect->h;
+
+  if(tmpwidth != last_w || tmpheight != last_h)
+  {
+   p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tmpwidth, tmpheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+   last_w = tmpwidth;
+   last_h = tmpheight;
+  }
+ }
+ else
+ {
+  bool ImageSizeChange = FALSE;
+
+  tmpwidth = uppow2(src_rect->w);
+  tmpheight = uppow2(src_rect->h);
+
+  // If the required GL texture size has changed, resize the texture! :b
+  if(tmpwidth != uppow2(last_w) || tmpheight != uppow2(last_h))
+  {
+   p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tmpwidth, tmpheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+   ImageSizeChange = TRUE;
+  }
+ 
+  // If the dimensions of our image stored in the texture have changed...
+  if(src_rect->w != last_w || src_rect->h != last_h)
+   ImageSizeChange = TRUE;
+
+  // Only clean up if we're using pixel shaders and/or bilinear interpolation
+  if(ImageSizeChange && (UsingShader || UsingIP))
+  {
+   uint32 neo_dbs = DummyBlackSize;
+
+   if(src_rect->w != tmpwidth && neo_dbs < src_rect->h)
+    neo_dbs = src_rect->h;
+
+   if(src_rect->h != tmpheight && neo_dbs < src_rect->w)
+    neo_dbs = src_rect->w;
+
+   if(neo_dbs != DummyBlackSize)
+   {
+    //printf("Realloc: %d\n", neo_dbs);
+    if(DummyBlack)
+     MDFN_free(DummyBlack);
+
+    if((DummyBlack = (uint32 *)MDFN_calloc(neo_dbs, sizeof(uint32), _("OpenGL dummy black texture data"))))
+     DummyBlackSize = neo_dbs;
+    else
+     DummyBlackSize = 0;
+   }
+
+   //printf("Cleanup: %d %d, %d %d\n", src_rect->w, src_rect->h, tmpwidth, tmpheight);
+
+   if(DummyBlack) // If memory allocation failed for some reason, don't clean the texture. :(
+   {
+    if(src_rect->w < tmpwidth)
+    {
+     //puts("X");
+     p_glPixelStorei(GL_UNPACK_ROW_LENGTH, 1);
+     p_glTexSubImage2D(GL_TEXTURE_2D, 0, src_rect->w, 0, 1, src_rect->h, GL_RGBA, GL_UNSIGNED_BYTE, DummyBlack);
+    }
+    if(src_rect->h < tmpheight)
+    {
+     //puts("Y");
+     p_glPixelStorei(GL_UNPACK_ROW_LENGTH, src_rect->w);
+     p_glTexSubImage2D(GL_TEXTURE_2D, 0, 0, src_rect->h, src_rect->w, 1, GL_RGBA, GL_UNSIGNED_BYTE, DummyBlack);
+    }
+   } // end if(DummyBlack)
+
+  }
+
+  last_w = src_rect->w;
+  last_h = src_rect->h;
+ }
+
  #if MDFN_WANT_OPENGL_SHADERS
  if(UsingShader)
   ShaderBegin(src_surface, src_rect, tmpwidth, tmpheight);
@@ -311,9 +393,51 @@ void FlipOpenGL(void)
 void KillOpenGL(void)
 {
  if(textures[0])
-  p_glDeleteTextures(3, &textures[0]);
- textures[0] = textures[1] = textures[2] = 0;
+  p_glDeleteTextures(4, &textures[0]);
+
+ textures[0] = textures[1] = textures[2] = textures[3] = 0;
+
+ if(rgb_mask)
+ {
+  p_glDeleteTextures(1, &rgb_mask);
+  rgb_mask = 0;
+ }
+
+ if(DummyBlack)
+ {
+  MDFN_free(DummyBlack);
+  DummyBlack = NULL;
+ }
+ DummyBlackSize = 0;
+
+ #if MDFN_WANT_OPENGL_SHADERS
+ if(UsingShader)
+ {
+  KillShader();
+  UsingShader = FALSE;
+ }
+ #endif
 }
+
+static bool CheckExtension(const char *extensions, const char *testval)
+{
+ const char *extparse = extensions;
+ const size_t testval_len = strlen(testval);
+
+ while((extparse = strstr(extparse, testval)))
+ {
+  if(extparse == extensions || *(extparse - 1) == ' ')
+  {
+   if(extparse[testval_len] == ' ' || extparse[testval_len] == 0)
+   {
+    return(TRUE);
+   }
+  }
+  extparse += testval_len;
+ }
+ return(FALSE);
+}
+
 /* Rectangle, left, right(not inclusive), top, bottom(not inclusive). */
 
 int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *screen)
@@ -363,14 +487,27 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
  MDFN_printf(_("OpenGL Implementation: %s %s %s\n"), p_glGetString(GL_VENDOR), p_glGetString(GL_RENDERER), p_glGetString(GL_VERSION));
  extensions=(const char*)p_glGetString(GL_EXTENSIONS);
 
+ MDFN_printf(_("Checking extensions:\n"));
+ MDFN_indent(1);
+
+ SupportNPOT = FALSE;
+
+ if(CheckExtension(extensions, "GL_ARB_texture_non_power_of_two"))
+ {
+  MDFN_printf(_("GL_ARB_texture_non_power_of_two found.\n"));
+  SupportNPOT = TRUE;
+ }
+ MDFN_indent(-1);
+
  p_glViewport(0, 0, screen->w, screen->h);
 
  p_glGenTextures(4, &textures[0]);
  p_glGenTextures(1, &rgb_mask);
  using_scanlines = 0;
 
- #if MDFN_WANT_OPENGL_SHADERS
  UsingShader = FALSE;
+
+ #if MDFN_WANT_OPENGL_SHADERS
  if(strcasecmp(pixshader.c_str(), "none"))
  {
   ShaderType pixshadertype;
@@ -419,10 +556,18 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
   }
   UsingShader = TRUE;
   ipolate = FALSE; // Disable texture interpolation, otherwise our pixel shaders will be extremely blurry.  
+  SupportNPOT = 0; // Our pixel shaders don't work right with NPOT textures:  FIXME
   p_glActiveTextureARB(GL_TEXTURE0_ARB);
  }
  #endif
 
+ // printf here because pixel shader code will set SupportNPOT to 0
+
+ if(SupportNPOT)
+  MDFN_printf(_("Using non-power-of-2 sized textures.\n"));
+ else
+  MDFN_printf(_("Using power-of-2 sized textures.\n"));
+
  if(scanlines)	// Check for scanlines, and disable them if vertical scaling isn't large enough.
  {
   int slcount;
@@ -464,6 +609,8 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
 
  p_glBindTexture(GL_TEXTURE_2D, textures[0]);
      
+ UsingIP = ipolate;
+
  p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,ipolate?GL_LINEAR:GL_NEAREST);
  p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,ipolate?GL_LINEAR:GL_NEAREST);
  p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
@@ -544,6 +691,9 @@ int InitOpenGL(int ipolate, int scanlines, std::string pixshader, SDL_Surface *s
   MaxTextureSize = 256;
  }
 
+ DummyBlack = NULL;
+ DummyBlackSize = 0;
+
  MDFN_indent(-1);
 
  return(1);
