@@ -15,9 +15,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
 
 #include "main.h"
 #include "video.h"
@@ -35,6 +33,7 @@
 #include "fps.h"
 #include "help.h"
 #include "video-state.h"
+#include "../video/selblur.h"
 
 typedef struct
 {
@@ -111,7 +110,6 @@ static SDL_Rect SMRect, SMDRect;
 
 
 static int curbpp;
-static int inited=0;
 
 static double exs,eys;
 static int exres, eyres;
@@ -132,7 +130,7 @@ void ClearBackBuffer(void)
 }
 
 /* Return 1 if video was killed, 0 otherwise(video wasn't initialized). */
-int KillVideo(int subsys)
+void KillVideo(void)
 {
  if(IconSurface)
  {
@@ -171,23 +169,11 @@ int KillVideo(int subsys)
   HelpSurface = NULL;
  }
 
- if(inited&1)
- {
-  if(cur_flags & SDL_OPENGL)
-   KillOpenGL();
-  else
-  {
-
-  }
-  inited&=~1;
-  return(1);
- }
- inited=0;
-
- SDL_WM_SetCaption("Mednafen","Mednafen");
+ if(cur_flags & SDL_OPENGL)
+  KillOpenGL();
 
  VideoGI = NULL;
- return(0);
+ cur_flags = 0;
 }
 
 static void GenerateRects(MDFN_Rect &DisplayRect)
@@ -280,6 +266,17 @@ int GetSpecialScalerID(const std::string &special_string)
  return(ret);
 }
 
+bool MDFND_ValidateVideoSetting(const char *name, const char *value)
+{
+ if(!strcasecmp(name, "vdriver"))
+ {
+  if(strcasecmp(value, "0") && strcasecmp(value, "1") && strcasecmp(value, "opengl") && strcasecmp(value, "sdl"))
+   return(FALSE);
+ }
+ return(TRUE);
+}
+
+
 bool MDFND_ValidateSpecialScalerSetting(const char *name, const char *value)
 {
  if(GetSpecialScalerID(value) < 0)
@@ -330,16 +327,6 @@ int InitVideo(MDFNGI *gi)
 
  CurrentScaler = _video.special ? &Scalers[_video.special - 1] : NULL;
 
- if(!(SDL_WasInit(SDL_INIT_VIDEO)&SDL_INIT_VIDEO))
-  if(SDL_InitSubSystem(SDL_INIT_VIDEO)==-1)
-  {
-   MDFND_PrintError(SDL_GetError());
-   MDFN_indent(-1);
-   return(0);
-  }
-
- inited|=1;
-
  vinf=SDL_GetVideoInfo();
 
  if(vinf->hw_available)
@@ -348,7 +335,18 @@ int InitVideo(MDFNGI *gi)
  if(_fullscreen)
   flags|=SDL_FULLSCREEN;
 
- unsigned int vdriver = MDFN_GetSettingUI("vdriver");
+ const std::string vdriverstr = MDFN_GetSettingS("vdriver");
+ unsigned int vdriver = 0;
+
+ if(!strcasecmp(vdriverstr.c_str(), "0") || !strcasecmp(vdriverstr.c_str(), "opengl"))
+  vdriver = 0;
+ else if(!strcasecmp(vdriverstr.c_str(), "1") || !strcasecmp(vdriverstr.c_str(), "sdl"))
+  vdriver = 1;
+ else
+ {
+  puts("Inconceivable!");
+  exit(-1);
+ }
 
  if(vdriver == 0)
  {
@@ -444,20 +442,18 @@ int InitVideo(MDFNGI *gi)
  if(!screen)
  {
   MDFND_PrintError(SDL_GetError());
-  KillVideo(1);
+  KillVideo();
   MDFN_indent(-1);
   return(0);
  }
  cur_flags = flags;
  curbpp = screen->format->BitsPerPixel;
 
- inited=1;
-
  MDFN_printf(_("Video Mode: %d x %d x %d bpp\n"),screen->w,screen->h,screen->format->BitsPerPixel);
  if(curbpp!=16 && curbpp!=24 && curbpp!=32)
  {
   MDFN_printf(_("Sorry, %dbpp modes are not supported by Mednafen.  Supported bit depths are 16bpp, 24bpp, and 32bpp.\n"),curbpp);
-  KillVideo(1);
+  KillVideo();
   MDFN_indent(-1);
   return(0);
  }
@@ -511,7 +507,7 @@ int InitVideo(MDFNGI *gi)
  {
   if(!InitOpenGL(evideoip, _video.scanlines, _video.pixshader, screen))
   {
-   KillVideo(1);
+   KillVideo();
    MDFN_indent(-1);
    return(0);
   }
@@ -570,7 +566,7 @@ int InitVideo(MDFNGI *gi)
    bs = 16;
    as = 24;
   }
- source_surface = SDL_CreateRGBSurfaceFrom(NULL, 256, 256, 32, gi->pitch, 0xFF << rs, 0xFF << gs, 0xFF << bs, 0xFF << as);
+ source_surface = SDL_CreateRGBSurfaceFrom(NULL, 256, 256, 32, gi->pitch, 0xFF << rs, 0xFF << gs, 0xFF << bs, 0);
 
  NetSurface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, screen->w, 18 * 5, 32, 0xFF << real_rs, 0xFF << real_gs, 0xFF << real_bs, 0xFF << real_as);
  SDL_SetColorKey(NetSurface, SDL_SRCCOLORKEY, 0);
@@ -682,45 +678,78 @@ static bool BlitInternalMessage(void)
 
 static void SubBlit(bool alpha_blend)
 {
+ SDL_Surface *eff_source_surface = source_surface;
+ SDL_Rect eff_src_rect = src_rect;
+ SDL_Surface *tmp_blur_surface = NULL;
+
+ // Handle selective blur first
+ if(0)
+ {
+  SelBlurImage sb_spec;
+
+  tmp_blur_surface = SDL_CreateRGBSurface(SDL_SWSURFACE, src_rect.w, src_rect.h, source_surface->format->BitsPerPixel, source_surface->format->Rmask, source_surface->format->Gmask, source_surface->format->Bmask, source_surface->format->Amask);
+  SDL_SetAlpha(tmp_blur_surface, 0, SDL_ALPHA_OPAQUE);
+
+  sb_spec.red_threshold = 8;
+  sb_spec.green_threshold = 7;
+  sb_spec.blue_threshold = 10;
+  sb_spec.radius = 3;
+  sb_spec.source = (uint32 *)source_surface->pixels + eff_src_rect.x + eff_src_rect.y * (source_surface->pitch / 4);
+  sb_spec.source_pitch32 = source_surface->pitch / 4;
+  sb_spec.dest = (uint32 *)tmp_blur_surface->pixels;
+  sb_spec.dest_pitch32 = tmp_blur_surface->pitch / 4;
+  sb_spec.width = eff_src_rect.w;
+  sb_spec.height = eff_src_rect.h;
+  sb_spec.red_shift = source_surface->format->Rshift;
+  sb_spec.green_shift = source_surface->format->Gshift;
+  sb_spec.blue_shift = source_surface->format->Bshift;
+
+  MDFN_SelBlur(&sb_spec);
+
+  eff_source_surface = tmp_blur_surface;
+  eff_src_rect.x = 0;
+  eff_src_rect.y = 0;
+ }
+
    if(CurrentScaler)
    {
     uint8 *screen_pixies;
     uint32 screen_pitch;
     SDL_Surface *bah_surface = NULL;
-    SDL_Rect boohoo_rect = src_rect;
+    SDL_Rect boohoo_rect = eff_src_rect;
 
     boohoo_rect.x = boohoo_rect.y = 0;
     boohoo_rect.w *= CurrentScaler->xscale;
     boohoo_rect.h *= CurrentScaler->yscale;
 
-    bah_surface = SDL_CreateRGBSurface(SDL_SWSURFACE, boohoo_rect.w, boohoo_rect.h, source_surface->format->BitsPerPixel, source_surface->format->Rmask, source_surface->format->Gmask, source_surface->format->Bmask, source_surface->format->Amask);
+    bah_surface = SDL_CreateRGBSurface(SDL_SWSURFACE, boohoo_rect.w, boohoo_rect.h, eff_source_surface->format->BitsPerPixel, eff_source_surface->format->Rmask, eff_source_surface->format->Gmask, eff_source_surface->format->Bmask, eff_source_surface->format->Amask);
 
     screen_pixies = (uint8 *)bah_surface->pixels;
     screen_pitch = bah_surface->pitch;
 
     if(CurrentScaler->id == NTVB_SCALE4X || CurrentScaler->id == NTVB_SCALE3X || CurrentScaler->id == NTVB_SCALE2X)
     {
-     uint8 *source_pixies = (uint8 *)source_surface->pixels + src_rect.x * source_surface->format->BytesPerPixel + src_rect.y * source_surface->pitch;
-     scale((CurrentScaler->id ==  NTVB_SCALE2X)?2:(CurrentScaler->id == NTVB_SCALE4X)?4:3, screen_pixies, screen_pitch, source_pixies, source_surface->pitch, source_surface->format->BytesPerPixel, src_rect.w, src_rect.h);
+     uint8 *source_pixies = (uint8 *)eff_source_surface->pixels + eff_src_rect.x * eff_source_surface->format->BytesPerPixel + eff_src_rect.y * eff_source_surface->pitch;
+     scale((CurrentScaler->id ==  NTVB_SCALE2X)?2:(CurrentScaler->id == NTVB_SCALE4X)?4:3, screen_pixies, screen_pitch, source_pixies, eff_source_surface->pitch, eff_source_surface->format->BytesPerPixel, eff_src_rect.w, eff_src_rect.h);
     }
     else if(CurrentScaler->id == NTVB_NN2X || CurrentScaler->id == NTVB_NN3X || CurrentScaler->id == NTVB_NN4X)
     {
-     nnx(CurrentScaler->id - NTVB_NN2X + 2, source_surface, &src_rect, bah_surface, &boohoo_rect);
+     nnx(CurrentScaler->id - NTVB_NN2X + 2, eff_source_surface, &eff_src_rect, bah_surface, &boohoo_rect);
     }
     else if(CurrentScaler->id == NTVB_NNY2X || CurrentScaler->id == NTVB_NNY3X || CurrentScaler->id == NTVB_NNY4X)
     {
-     nnyx(CurrentScaler->id - NTVB_NNY2X + 2, source_surface, &src_rect, bah_surface, &boohoo_rect);
+     nnyx(CurrentScaler->id - NTVB_NNY2X + 2, eff_source_surface, &eff_src_rect, bah_surface, &boohoo_rect);
     }
     else 
     {
-     uint8 *source_pixies = (uint8 *)source_surface->pixels + src_rect.x * source_surface->format->BytesPerPixel + src_rect.y * source_surface->pitch;
+     uint8 *source_pixies = (uint8 *)eff_source_surface->pixels + eff_src_rect.x * eff_source_surface->format->BytesPerPixel + eff_src_rect.y * eff_source_surface->pitch;
 
      if(CurrentScaler->id == NTVB_HQ2X)
-      hq2x_32(source_pixies, screen_pixies, src_rect.w, src_rect.h, source_surface->pitch, screen_pitch);
+      hq2x_32(source_pixies, screen_pixies, eff_src_rect.w, eff_src_rect.h, eff_source_surface->pitch, screen_pitch);
      else if(CurrentScaler->id == NTVB_HQ3X)
-      hq3x_32(source_pixies, screen_pixies, src_rect.w, src_rect.h, source_surface->pitch, screen_pitch);
+      hq3x_32(source_pixies, screen_pixies, eff_src_rect.w, eff_src_rect.h, eff_source_surface->pitch, screen_pitch);
      else if(CurrentScaler->id == NTVB_HQ4X)
-      hq4x_32(source_pixies, screen_pixies, src_rect.w, src_rect.h, source_surface->pitch, screen_pitch);
+      hq4x_32(source_pixies, screen_pixies, eff_src_rect.w, eff_src_rect.h, eff_source_surface->pitch, screen_pitch);
 
      if(bah_surface->format->Rshift != real_rs || bah_surface->format->Gshift != real_gs ||
   	bah_surface->format->Bshift != real_bs)
@@ -746,19 +775,24 @@ static void SubBlit(bool alpha_blend)
     }
 
     if(cur_flags & SDL_OPENGL)
-     BlitOpenGL(bah_surface, &boohoo_rect, &dest_rect, &src_rect, alpha_blend);
+     BlitOpenGL(bah_surface, &boohoo_rect, &dest_rect, &eff_src_rect, alpha_blend);
     else
-     BlitNonGL(bah_surface, &boohoo_rect, &src_rect, &dest_rect, screen, _video.scanlines, CurGame->rotated);
+     BlitNonGL(bah_surface, &boohoo_rect, &eff_src_rect, &dest_rect, screen, _video.scanlines, CurGame->rotated);
 
     SDL_FreeSurface(bah_surface);
    }
    else // No special scaler:
    {
     if(cur_flags & SDL_OPENGL)
-     BlitOpenGL(source_surface, &src_rect, &dest_rect, &src_rect, alpha_blend);
+     BlitOpenGL(eff_source_surface, &eff_src_rect, &dest_rect, &eff_src_rect, alpha_blend);
     else
-     BlitNonGL(source_surface, &src_rect, &src_rect, &dest_rect, screen, _video.scanlines, CurGame->rotated);
+     BlitNonGL(eff_source_surface, &eff_src_rect, &eff_src_rect, &dest_rect, screen, _video.scanlines, CurGame->rotated);
    }
+
+ if(tmp_blur_surface)
+ {
+  SDL_FreeSurface(tmp_blur_surface);
+ } 
 }
 
 void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths)
