@@ -16,12 +16,6 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <memory.h>
-#include <stdarg.h>
-#include <string.h>
-
 #include "GBA.h"
 #include "../general.h"
 #include "../player.h"
@@ -31,6 +25,7 @@
 #include "../netplay.h"
 #include "../mempatcher.h"
 #include "../md5.h"
+#include "../endian.h"
 #include "GBAinline.h"
 #include "Globals.h"
 #include "Gfx.h"
@@ -39,12 +34,19 @@
 #include "Sound.h"
 #include "sram.h"
 #include "bios.h"
-#include "elf.h"
 #include "Port.h"
 
 #include "arm.h"
 #include "thumb.h"
 
+#include <memory.h>
+#include <stdarg.h>
+#include <string.h>
+#include <trio/trio.h>
+
+static bool CPUInit(const std::string bios_fn);
+static void CPUReset(void);
+
 #define UPDATE_REG(address, value)\
   {\
     WRITE16LE(((uint16 *)&ioMem[address]),value);\
@@ -70,16 +72,16 @@ int dummyAddress = 0;
 bool8 cpuBreakLoop = false;
 int cpuNextEvent = 0;
 
-static int gbaSaveType = 0; // used to remember the save type on reset
 static bool8 intState = false;
 static bool8 stopState = false;
 static bool8 holdState = false;
 static int holdType = 0;
 
-bool8 cpuSramEnabled = true;
-bool8 cpuFlashEnabled = true;
-bool8 cpuEEPROMEnabled = true;
-bool8 cpuEEPROMSensorEnabled = false;
+static bool FlashSizeSet; // Set to TRUE if explicitly set by the user
+bool8 cpuSramEnabled;
+bool8 cpuFlashEnabled;
+bool8 cpuEEPROMEnabled;
+bool8 cpuEEPROMSensorEnabled;
 
 uint32 cpuPrefetch[2];
 
@@ -91,7 +93,6 @@ GBATimer timers[4];
 
 uint32 dmaSource[4] = {0};
 uint32 dmaDest[4] = {0};
-void (*cpuSaveGameFunc)(uint32,uint8) = flashSaveDecide;
 void (*renderLine)() = mode0RenderLine;
 bool8 fxOn = false;
 bool8 windowOn = false;
@@ -133,11 +134,7 @@ uint8 memoryWaitSeq32[16] =
 
 uint8 biosProtected[4];
 
-#ifndef LSB_FIRST
-bool8 cpuBiosSwapped = false;
-#endif
-
-static uint32 myROM[] = 
+static const uint32 myROM[] = 
 {
  #include "myrom.h"
 };
@@ -232,7 +229,6 @@ void CPUUpdateRenderBuffers(bool8 force)
 }
 
 extern SFORMAT eepromSaveData[];
-extern SFORMAT flashSaveData[];
 
 static void pre_post_save(void)
 {
@@ -311,12 +307,12 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFVAR(COLEV),
   SFVAR(COLY),
 
-  SFARRAY32(DMSAD_L, 4),
-  SFARRAY32(DMSAD_H, 4),
-  SFARRAY32(DMDAD_L, 4),
-  SFARRAY32(DMDAD_H, 4),
-  SFARRAY32(DMCNT_L, 4),
-  SFARRAY32(DMCNT_H, 4),
+  SFARRAY16(DMSAD_L, 4),
+  SFARRAY16(DMSAD_H, 4),
+  SFARRAY16(DMDAD_L, 4),
+  SFARRAY16(DMDAD_H, 4),
+  SFARRAY16(DMCNT_L, 4),
+  SFARRAY16(DMCNT_H, 4),
 
   SFVAR(timers[0].D),
   SFVAR(timers[0].CNT),
@@ -370,7 +366,12 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFVAR(armIrqEnable),
   SFVAR(armNextPC),
   SFVAR(armMode),
-  SFVAR(saveType),
+
+  SFVAR(cpuSramEnabled),
+  SFVAR(cpuFlashEnabled),
+  SFVAR(cpuEEPROMEnabled),
+  SFVAR(cpuEEPROMSensorEnabled),
+
   SFEND
  };
 
@@ -394,10 +395,13 @@ static int StateAction(StateMem *sm, int load, int data_only)
  if(!MDFNSS_StateAction(sm, load, data_only, RAMState, "RAM"))
   return(0);
 
- if(!MDFNSS_StateAction(sm, load, data_only, eepromSaveData, "EEPR"))
-  return(0);
+ if(cpuEEPROMEnabled)
+ {
+  if(!MDFNSS_StateAction(sm, load, data_only, eepromSaveData, "EEPR"))
+   return(0);
+ }
 
- if(!MDFNSS_StateAction(sm, load, data_only, flashSaveData, "FLSH"))
+ if(!GBA_Flash_StateAction(sm, load, data_only))
   return(0);
 
  if(!load || load >= 0x500)
@@ -420,33 +424,6 @@ static int StateAction(StateMem *sm, int load, int data_only)
   CPUUpdateWindow0();
   CPUUpdateWindow1();
 
-  gbaSaveType = 0;
-  saveType = 0;
-
-  switch(saveType) 
-  {
-  case 0:
-    cpuSaveGameFunc = flashSaveDecide;
-    break;
-  case 1:
-    cpuSaveGameFunc = sramWrite;
-    gbaSaveType = 1;
-    break;
-  case 2:
-    cpuSaveGameFunc = flashWrite;
-    gbaSaveType = 2;
-    break;
-  case 5:
-    gbaSaveType = 5;
-    break;
-  default:
-    MDFN_PrintError(_("Unsupported save type %d"), saveType);
-    break;
-  }
-
-  if(eepromInUse)
-    gbaSaveType = 3;
-
   if(armState) {
     ARM_PREFETCH;
   } else {
@@ -457,121 +434,79 @@ static int StateAction(StateMem *sm, int load, int data_only)
  return(1);
 }
 
-bool8 CPUWriteBatteryFile(const char *fileName)
+bool8 CPUWriteBatteryFile(const char *filename)
 {
-  if(gbaSaveType == 0) {
-    if(eepromInUse)
-      gbaSaveType = 3;
-    else switch(saveType) {
-    case 1:
-      gbaSaveType = 1;
-      break;
-    case 2:
-      gbaSaveType = 2;
-      break;
-    }
+ if(cpuSramEnabled || cpuFlashEnabled)
+ {
+  if(cpuSramEnabled)
+  {
+   if(!MDFN_DumpToFile(filename, 6, flashSaveMemory, 0x10000))
+    return(0);
   }
-  
-  if((gbaSaveType) && (gbaSaveType!=5)) {
-    FILE *file = fopen(fileName, "wb");
-    
-    if(!file) {
-      MDFN_PrintError(_("Error creating file %s"), fileName);
-      return false;
-    }
-    
-    // only save if Flash/Sram in use or EEprom in use
-    if(gbaSaveType != 3) {
-      if(gbaSaveType == 2) {
-        if(fwrite(flashSaveMemory, 1, flashSize, file) != (size_t)flashSize) {
-          fclose(file);
-          return false;
-        }
-      } else {
-        if(fwrite(flashSaveMemory, 1, 0x10000, file) != 0x10000) {
-          fclose(file);
-          return false;
-        }
-      }
-    } else {
-      if(fwrite(eepromData, 1, eepromSize, file) != (size_t)eepromSize) {
-        fclose(file);
-        return false;
-      }
-    }
-    fclose(file);
+  else if(cpuFlashEnabled)
+  {
+   if(!MDFN_DumpToFile(filename, 6, flashSaveMemory, flashSize))
+    return(0);
   }
-  return true;
+  return(TRUE);
+ }
+ return(FALSE);
 }
 
-bool8 CPUReadBatteryFile(const char *fileName)
+bool8 CPUReadBatteryFile(const char *filename)
 {
-  FILE *file = fopen(fileName, "rb");
+ gzFile gp = gzopen(filename, "rb");
+ long size;
 
-  if(!file)
-    return false;
-  
-  // check file size to know what we should read
-  fseek(file, 0, SEEK_END);
+ if(!gp)
+  return(FALSE);
 
-  long size = ftell(file);
-  fseek(file, 0, SEEK_SET);
+ size = 0;
+ while(gzgetc(gp) != EOF)
+  size++;
 
-  if(size == 512 || size == 0x2000) {
-    if(fread(eepromData, 1, size, file) != (size_t)size) {
-      fclose(file);
-      return false;
-    }
-  } else {
-    if(size == 0x20000) {
-      if(fread(flashSaveMemory, 1, 0x20000, file) != 0x20000) {
-        fclose(file);
-        return false;
-      }
-      flashSetSize(0x20000);
-    } else {
-      if(fread(flashSaveMemory, 1, 0x10000, file) != 0x10000) {
-        fclose(file);
-        return false;
-      }
-      flashSetSize(0x10000);
-    }
+ if(size == 512 || size == 0x2000) // Load the file as EEPROM for Mednafen < 0.8.2
+ {
+  gzclose(gp);
+
+  if(cpuEEPROMEnabled)
+  {
+   puts("note:  Loading sav file as eeprom data");
+   GBA_EEPROM_LoadFile(filename);
   }
-  fclose(file);
-  return true;
-}
+  return(FALSE);
+ }
 
-bool8 CPUIsGBABios(const char * file)
-{
-  if(strlen(file) > 4) {
-    char * p = strrchr(file,'.');
+ gzseek(gp, 0, SEEK_SET);
 
-    if(p != NULL) {
-      if(_stricmp(p, ".gba") == 0)
-        return true;
-      if(_stricmp(p, ".agb") == 0)
-        return true;
-      if(_stricmp(p, ".bin") == 0)
-        return true;
-      if(_stricmp(p, ".bios") == 0)
-        return true;
-    }
+ if(size == 0x20000) 
+ {
+  if(gzread(gp, flashSaveMemory, 0x20000) != 0x20000) 
+  {
+   gzclose(gp);
+   return(FALSE);
   }
-  
-  return false;
-}
-
-bool8 CPUIsELF(const char *file)
-{
-  if(strlen(file) > 4) {
-    char * p = strrchr(file,'.');
-    
-    if(p != NULL) {
-      if(_stricmp(p, ".elf") == 0)
-        return true;
-    }
+  if(!FlashSizeSet)
+  {
+   flashSetSize(0x20000);
+   FlashSizeSet = TRUE;
   }
-  return false;
+ } 
+ else 
+ {
+  if(gzread(gp, flashSaveMemory, 0x10000) != 0x10000) 
+  {
+   gzclose(gp);
+   return(FALSE);
+  }
+  if(!FlashSizeSet)
+  {
+   flashSetSize(0x10000);
+   FlashSizeSet = TRUE;
+  }
+ }
+ gzclose(gp);
+ return(TRUE);
 }
 
 void CPUCleanUp()
@@ -627,7 +562,7 @@ void CPUCleanUp()
    systemColorMap32 = NULL;
   }
 
-  elfCleanUp();
+ GBA_Flash_Kill();
 }
 
 #include "../psf.h"
@@ -638,8 +573,11 @@ static void CloseGame(void)
  if(!pi)
  {
   MDFN_FlushGameCheats(0);
+  GBA_EEPROM_SaveFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "eep").c_str());
   CPUWriteBatteryFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str());
  }
+
+ // Call CPUCleanUp() to deallocate memory AFTER the backup memory is saved.
  CPUCleanUp();
 
  if(pi)
@@ -703,10 +641,30 @@ static void RedoColorMap(int rshift, int gshift, int bshift)
  }
 }
 
+static bool TestMagic(const char *name, MDFNFILE *fp)
+{
+ if(!memcmp(fp->data, "PSF\x22", 4))
+  return(TRUE);
+
+ if(!strcasecmp(fp->ext, "gba") || !strcasecmp(fp->ext, "agb"))
+  return(TRUE);
+
+ if(fp->size >= 192 && !strcasecmp(fp->ext, "bin"))
+ {
+  if((fp->data[0xb2] == 0x96 && fp->data[0xb3] == 0x00) || (fp->data[0] == 0x2E && fp->data[3] == 0xEA))
+   return(TRUE);
+ }
+
+ return(FALSE);
+}
+
 static int Load(const char *name, MDFNFILE *fp)
 {
   int size = 0x2000000;
 
+  if(!TestMagic(name, fp))
+   return(-1);
+
   layerSettings = 0xFF00;
   if(!memcmp(fp->data, "PSF\x22", 4))
   {
@@ -722,14 +680,9 @@ static int Load(const char *name, MDFNFILE *fp)
    static UTF8 *sn;
    sn = (UTF8*)pi->title;
    Player_Init(1, (UTF8*)pi->game, (UTF8*)pi->artist, (UTF8*)pi->copyright, &sn);
-   //saveType = 2;
   }
   else
   {
-   if(fp->size < 192) return(-1);
-   if((fp->data[0xb2] == 0x96 && fp->data[0xb3] == 0x00) || (fp->data[0] == 0x2E && fp->data[3] == 0xEA) || !strcasecmp(fp->ext, "gba") ||
-	!strcasecmp(fp->ext, "agb"))
-   {
     rom = (uint8 *)malloc(0x2000000);
     workRAM = (uint8 *)calloc(1, 0x40000);
 
@@ -754,11 +707,6 @@ static int Load(const char *name, MDFNFILE *fp)
       WRITE16LE(temp, (i >> 1) & 0xFFFF);
       temp++;
     }
-   }
-   else
-   {
-    return(-1);
-   }
   }
   bios = (uint8 *)calloc(1,0x4000);
   if(bios == NULL) {
@@ -833,14 +781,26 @@ static int Load(const char *name, MDFNFILE *fp)
    MDFN_LoadGameCheats(0);
   }
 
-  CPUInit(NULL, 0);
+  if(!CPUInit(MDFN_GetSettingS("gba.bios")))
+  {
+   CPUCleanUp();
+   return(0);
+  }
   CPUReset();
 
   GBA_Flash_Init();
   eepromInit();
 
   if(!pi)
-   CPUReadBatteryFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str());
+  {
+   // EEPROM might be loaded from within CPUReadBatteryFile for support for Mednafen < 0.8.2, so call before GBA_EEPROM_LoadFile(), which
+   // is more specific...kinda.
+   if(cpuSramEnabled || cpuFlashEnabled)
+    CPUReadBatteryFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str());
+
+   if(cpuEEPROMEnabled)
+    GBA_EEPROM_LoadFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "eep").c_str());
+  }
 
   std::string colormap_fn = MDFN_GetSettingS("gba.colormap");
 
@@ -864,8 +824,7 @@ static int Load(const char *name, MDFNFILE *fp)
     }
    }
   }
-
-  return size;
+ return(1);
 }
 
 void doMirroring (bool8 b)
@@ -2214,9 +2173,80 @@ void applyTimer ()
   timerOnOffDelay = 0;
 }
 
+static void FLASH_SRAM_Write(uint32 A, uint32 V)
+{
+// printf("%08x %08x\n", A, V);
+
+ if(cpuFlashEnabled && cpuSramEnabled)
+ {
+  if((A & 0xFFFF) == 0x5555 && (V & 0xFF) == 0xAA)
+   cpuSramEnabled = FALSE;
+  else if((A & 0xFFFF) != 0x2AAA)
+   cpuFlashEnabled = FALSE;
+
+  if(!cpuFlashEnabled || !cpuSramEnabled)
+   printf("%s emulation disabled by write to:  %08x %08x\n", cpuSramEnabled ? "FLASH" : "SRAM", A, V);
+ }
+
+ if(cpuSramEnabled)
+  sramWrite(A, V);
+
+ if(cpuFlashEnabled)
+ flashWrite(A, V);
+}
+
+void CPUWriteMemory(uint32 address, uint32 value)
+{
+ switch(address >> 24) 
+ {
+  case 0x02:
+      WRITE32LE(((uint32 *)&workRAM[address & 0x3FFFC]), value);
+    break;      \
+  case 0x03:    \
+      WRITE32LE(((uint32 *)&internalRAM[address & 0x7ffC]), value);     \
+    break;      \
+  case 0x04:    \
+    if(address < 0x4000400) {   \
+      CPUUpdateRegister((address & 0x3FC), value & 0xFFFF);     \
+      CPUUpdateRegister((address & 0x3FC) + 2, (value >> 16));  \
+    }   \
+    break;      \
+  case 0x05:    \
+    WRITE32LE(((uint32 *)&paletteRAM[address & 0x3FC]), value); \
+    break;      \
+  case 0x06:    \
+    address = (address & 0x1fffc);
+    if (((DISPCNT & 7) >2) && ((address & 0x1C000) == 0x18000))
+     return;
+    if ((address & 0x18000) == 0x18000)
+     address &= 0x17fff;
+    WRITE32LE(((uint32 *)&vram[address]), value);
+    break;      \
+
+  case 0x07:
+    WRITE32LE(((uint32 *)&oam[address & 0x3fc]), value);
+    break;
+
+  case 0x0D:
+    if(cpuEEPROMEnabled) 
+    {
+     eepromWrite(address, value);
+     break;
+    }
+    break;
+
+  case 0x0E:
+    FLASH_SRAM_Write(address, value);
+    break;
+
+  default: break;
+ }
+}
+
 void CPUWriteHalfWord(uint32 address, uint16 value)
 {
-  switch(address >> 24) {
+ switch(address >> 24) 
+ {
   case 2:
       WRITE16LE(((uint16 *)&workRAM[address & 0x3FFFE]),value);
     break;
@@ -2250,32 +2280,35 @@ void CPUWriteHalfWord(uint32 address, uint16 value)
     } else goto unwritable;
     break;
   case 13:
-    if(cpuEEPROMEnabled) {
-      eepromWrite(address, (uint8)value);
-      break;
-    }
-    goto unwritable;
-  case 14:
-    if(!eepromInUse | cpuSramEnabled | cpuFlashEnabled) {
-      (*cpuSaveGameFunc)(address, (uint8)value);
-      break;
+    if(cpuEEPROMEnabled) 
+    {
+     eepromWrite(address, (uint8)value);
+     break;
     }
     goto unwritable;
+
+  case 0x0E:
+    FLASH_SRAM_Write(address, value);
+    break;
+
   default:
   unwritable:
     break;
-  }
+ }
 }
 
 void CPUWriteByte(uint32 address, uint8 b)
 {
-  switch(address >> 24) {
+ switch(address >> 24) 
+ {
   case 2:
-        workRAM[address & 0x3FFFF] = b;
-    break;
+      workRAM[address & 0x3FFFF] = b;
+      break;
+
   case 3:
       internalRAM[address & 0x7fff] = b;
-    break;
+      break;
+
   case 4:
     if(address < 0x4000400) 
     {
@@ -2364,60 +2397,126 @@ void CPUWriteByte(uint32 address, uint8 b)
     // byte writes to OAM are ignored
     //    *((uint16 *)&oam[address & 0x3FE]) = (b << 8) | b;
     break;    
-  case 13:
+
+  case 0xD:
     if(cpuEEPROMEnabled) {
       eepromWrite(address, b);
       break;
     }
     goto unwritable;
-  case 14:
-    if (!(saveType == 5) && (!eepromInUse | cpuSramEnabled | cpuFlashEnabled)) {
-      (*cpuSaveGameFunc)(address, b);
-      break;
-    }
+
+  case 0xE:
+    FLASH_SRAM_Write(address, b);
+    break;
+
     // default
   default:
   unwritable:
     break;
-  }
+ }
 }
 
 uint8 cpuBitsSet[256];
 uint8 cpuLowestBitSet[256];
 
-void CPUInit(const char *biosFileName, bool8 useBiosFile)
+static bool CPUInit(const std::string bios_fn)
 {
-#ifndef LSB_FIRST
-  if(!cpuBiosSwapped) {
-    for(unsigned int i = 0; i < sizeof(myROM)/4; i++) {
-      WRITE32LE(&myROM[i], myROM[i]);
+ FILE *memfp;
+
+ FlashSizeSet = FALSE;
+
+ memfp = fopen(MDFN_MakeFName(MDFNMKF_SAV, 0, "type").c_str(), "rb");
+ if(memfp)
+ {
+  char linebuffer[256];
+
+  cpuSramEnabled = FALSE;
+  cpuFlashEnabled = FALSE;
+  cpuEEPROMEnabled = FALSE;
+  cpuEEPROMSensorEnabled = FALSE;
+
+  while(fgets(linebuffer, 256, memfp))
+  {
+   char args[2][256];
+   int acount = trio_sscanf(linebuffer, "%.255s %.255s", args[0], args[1]);
+
+   if(acount < 1)
+    continue;
+
+   if(!strcasecmp(args[0], "sram"))
+    cpuSramEnabled = TRUE;
+   else if(!strcasecmp(args[0], "flash"))
+   {
+    cpuFlashEnabled = TRUE;
+    if(acount == 2)
+    {
+     int size_temp = atoi(args[1]);
+     if(size_temp == 0x10000 || size_temp == 0x20000)
+     {
+      flashSetSize(size_temp);
+      FlashSizeSet = TRUE;
+     }
+     else if(size_temp == 64 || size_temp == 128)
+     {
+      flashSetSize(size_temp * 1024);
+      FlashSizeSet = TRUE;
+     }
+     else
+      puts("Flash size error");
     }
-    cpuBiosSwapped = true;
+   }
+   else if(!strcasecmp(args[0], "eeprom"))
+    cpuEEPROMEnabled = TRUE;
+   else if(!strcasecmp(args[0], "sensor"))
+    cpuEEPROMSensorEnabled = TRUE;
   }
-#endif
-  gbaSaveType = 0;
-  eepromInUse = 0;
-  saveType = 0;
-  useBios = false;
+  fclose(memfp);
+ }
+ else
+ {
+  cpuSramEnabled = true;
+  cpuFlashEnabled = true;
+  cpuEEPROMEnabled = true;
+  cpuEEPROMSensorEnabled = true;
+ }
+
+ useBios = false;
   
-  #ifdef MYOO
-  if(useBiosFile) {
-    int size = 0x4000;
-    if(utilLoad(biosFileName,
-                CPUIsGBABios,
-                bios,
-                size)) {
-      if(size == 0x4000)
-        useBios = true;
-      else
-        MDFN_PrintError(_("Invalid BIOS file size"));
-    }
+ if(bios_fn != "" && bios_fn != "0" && bios_fn != "none")
+ {
+  MDFNFILE *bios_fp;
+  bios_fp = MDFN_fopen(bios_fn.c_str(), NULL, "rb", ".gba\0.agb\0.bin\0.bios\0");
+
+  if(!bios_fp)
+  {
+   MDFN_PrintError(_("Error opening \"%s\""), bios_fn.c_str());
+   return(0);
   }
-  #endif
-  if(!useBios) {
-    memcpy(bios, myROM, sizeof(myROM));
+  
+  if(MDFN_fgetsize(bios_fp) != 0x4000)
+  {
+   MDFN_PrintError(_("Invalid BIOS file size"));
+   MDFN_fclose(bios_fp);
+   return(0);
   }
 
+  if(MDFN_fread(bios, 1, 0x4000, bios_fp) != 0x4000)
+  {
+   MDFN_PrintError(_("Error reading BIOS file"));
+   MDFN_fclose(bios_fp);
+   return(0);
+  }
+
+  MDFN_fclose(bios_fp);
+  useBios = true;
+ }
+
+ if(!useBios)
+ {
+  memcpy(bios, myROM, sizeof(myROM));
+  Endian_A32_NE_to_LE(bios, sizeof(myROM) / 4);
+ }
+
   int i = 0;
 
   biosProtected[0] = 0x00;
@@ -2480,23 +2579,11 @@ void CPUInit(const char *biosFileName, bool8 useBiosFile)
   } else {
 
   }
+ return(1);
 }
 
-void CPUReset()
+static void CPUReset()
 {
-  if(gbaSaveType == 0) {
-    if(eepromInUse)
-      gbaSaveType = 3;
-    else
-      switch(saveType) {
-      case 1:
-        gbaSaveType = 1;
-        break;
-      case 2:
-        gbaSaveType = 2;
-        break;
-      }
-  }
   rtcReset();
   // clean registers
   memset(&reg[0], 0, sizeof(reg));
@@ -2637,7 +2724,7 @@ void CPUReset()
   dmaDest[2] = 0;
   dmaSource[3] = 0;
   dmaDest[3] = 0;
-  cpuSaveGameFunc = flashSaveDecide;
+
   renderLine = mode0RenderLine;
   fxOn = false;
   windowOn = false;
@@ -2695,52 +2782,6 @@ void CPUReset()
       BIOS_RegisterRamReset(0xfe);
   }
 
-  switch(cpuSaveType) {
-  case 0: // automatic
-    cpuSramEnabled = true;
-    cpuFlashEnabled = true;
-    cpuEEPROMEnabled = true;
-    cpuEEPROMSensorEnabled = false;
-    break;
-  case 1: // EEPROM
-    cpuSramEnabled = false;
-    cpuFlashEnabled = false;
-    cpuEEPROMEnabled = true;
-    cpuEEPROMSensorEnabled = false;
-    // EEPROM usage is automatically detected
-    break;
-  case 2: // SRAM
-    cpuSramEnabled = true;
-    cpuFlashEnabled = false;
-    cpuEEPROMEnabled = false;
-    cpuEEPROMSensorEnabled = false;
-    cpuSaveGameFunc = sramDelayedWrite; // to insure we detect the write
-    break;
-  case 3: // FLASH
-    cpuSramEnabled = false;
-    cpuFlashEnabled = true;
-    cpuEEPROMEnabled = false;
-    cpuEEPROMSensorEnabled = false;
-    cpuSaveGameFunc = flashDelayedWrite; // to insure we detect the write
-    break;
-  case 4: // EEPROM+Sensor
-    cpuSramEnabled = false;
-    cpuFlashEnabled = false;
-    cpuEEPROMEnabled = true;
-    cpuEEPROMSensorEnabled = true;
-    // EEPROM usage is automatically detected
-    break;
-  case 5: // NONE
-    cpuSramEnabled = false;
-    cpuFlashEnabled = false;
-    cpuEEPROMEnabled = false;
-    cpuEEPROMSensorEnabled = false;
-    // no save at all
-    break;
-  } 
-
-  saveType = gbaSaveType = cpuSaveType;
-
   ARM_PREFETCH;
 
   cpuDmaHack = false;
@@ -3136,24 +3177,14 @@ void CPULoop(int ticks)
   }
 }
 
-static int needrew = 0;
-
-static void DoRewind(void)
-{
- needrew = 1;
-}
-
-static void Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int32 *SoundBufSize, int skip)
+static void Emulate(EmulateSpecStruct *espec)
 {
  padbufblah = padq[0] | (padq[1] << 8);
 
- int didrew = MDFN_StateEvil(needrew);
- needrew = 0;
-
- MDFNGameInfo->fb = pXBuf;
+ MDFNGameInfo->fb = espec->pixels;
  frameready = 0;
 
- HelloSkipper = skip;
+ HelloSkipper = espec->skip;
 
  if(pi) HelloSkipper = 1;
 
@@ -3163,9 +3194,9 @@ static void Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int3
  while(!frameready && (soundTS < 300000))
   CPULoop(300000);
 
- *SoundBuf = MDFNGBASOUND_Flush(SoundBufSize, didrew);
+ *(espec->SoundBuf) = MDFNGBASOUND_Flush(espec->SoundBufSize);
  if(pi)
-  Player_Draw(pXBuf, 0, *SoundBuf, *SoundBufSize);
+  Player_Draw(espec->pixels, 0, *(espec->SoundBuf), *(espec->SoundBufSize));
 }
 
 static bool ToggleLayer(int which)
@@ -3192,6 +3223,7 @@ static void DoSimpleCommand(int cmd)
 
 static MDFNSetting GBASettings[] =
 {
+ { "gba.bios", gettext_noop("Path to optional GBA BIOS ROM image."), MDFNST_STRING, "" },
  { "gba.forcemono", gettext_noop("Force monophonic sound output."), MDFNST_BOOL, "0" },
  { "gba.colormap", gettext_noop("Load custom color map from specified file."), MDFNST_STRING, "" },
  { NULL }
@@ -3249,7 +3281,6 @@ MDFNGI EmulatedGBA =
  NULL,
  NULL,
  StateAction,
- DoRewind,
  Emulate,
  SetPixelFormat,
  MDFNGBA_SetInput,
