@@ -21,17 +21,18 @@
 #define VBA_FLASH_H
 
 bool GBA_Flash_Init(void);
-void GBA_Flash_Close(void);
+void GBA_Flash_Kill(void);
 void GBA_Flash_Reset(void);
 
 extern void flashSaveGame(gzFile gzFile);
 extern void flashReadGame(gzFile gzFile, int version);
 extern uint8 flashRead(uint32 address);
 extern void flashWrite(uint32 address, uint8 byte);
-extern void flashDelayedWrite(uint32 address, uint8 byte);
 extern uint8 *flashSaveMemory;
-extern void flashSaveDecide(uint32 address, uint8 byte);
 extern void flashSetSize(int size);
 
 extern int flashSize;
+
+int GBA_Flash_StateAction(StateMem *sm, int load, int data_only);
+
 #endif // VBA_FLASH_H
