@@ -49,8 +49,6 @@
 //#include <crtdbg.h>
 //#define TRACE_SUSIE
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include "system.h"
 #include "susie.h"
