@@ -15,19 +15,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include        "mednafen.h"
+
 #include        <string.h>
-#include	<stdio.h>
-#include	<stdlib.h>
 #include	<stdarg.h>
 #include	<errno.h>
 #include	<sys/types.h>
 #include	<sys/stat.h>
 #include	<unistd.h>
 #include	<trio/trio.h>
-#include	<assert.h>
 
-#include	"types.h"
-#include	"mednafen.h"
 #include	"netplay.h"
 #include	"netplay-driver.h"
 #include	"general.h"
@@ -39,10 +36,11 @@
 #include        "video.h"
 #include	"file.h"
 #include	"wave.h"
-#include	"cdromif.h"
+#include	"cdrom/cdromif.h"
 #include	"mempatcher.h"
-#include	"minilzo.h"
+#include	"compress/minilzo.h"
 #include	"tests.h"
+#include	"video/vblur.h"
 
 static const char *CSD_vblur = gettext_noop("Blur each frame with the last frame.");
 static const char *CSD_vblur_accum = gettext_noop("Accumulate color data rather than discarding it.");
@@ -52,8 +50,8 @@ static bool ValidateSetting(const char *name, const char *value)
 {
  if(!strcasecmp(name, "srwcompressor"))
  {
-  // If it doesn't match "minilzo" or "quicklz", error out!
-  if(strcasecmp(value, "minilzo") && strcasecmp(value, "quicklz"))
+  // If it doesn't match "minilzo", "quicklz", or "blz", error out!
+  if(strcasecmp(value, "minilzo") && strcasecmp(value, "quicklz") && strcasecmp(value, "blz"))
    return(FALSE);
  }
 
@@ -62,9 +60,15 @@ static bool ValidateSetting(const char *name, const char *value)
 
 static MDFNSetting MednafenSettings[] =
 {
-  { "srwcompressor", gettext_noop("Compressor to use with state rewinding:  \"minilzo\" or \"quicklz\""), MDFNST_STRING, "minilzo", NULL, NULL, ValidateSetting },
+  { "srwcompressor", gettext_noop("Compressor to use with state rewinding:  \"minilzo\", \"quicklz\", or \"blz\""), MDFNST_STRING, "minilzo", NULL, NULL, ValidateSetting },
+  { "srwframes", gettext_noop("Number of frames to keep states for when state rewinding is enabled."), MDFNST_UINT, "600", "10", "99999" },
   { "snapname", gettext_noop("If value is true, use an alternate naming scheme(file base and numeric) for screen snapshots."), MDFNST_BOOL, "0"},
   { "dfmd5", gettext_noop("Include the MD5 hash of the loaded game in the filenames of the data file(save states, SRAM backups) Mednafen creates."), MDFNST_BOOL, "1" },
+
+  #ifdef NEED_CDEMU
+  { "cdrom.lec_eval", gettext_noop("Enable simple error correction of raw data sector rips by evaluating L-EC and EDC data."), MDFNST_BOOL, "1" },
+  #endif
+
   { "path_snap", gettext_noop("Path override for screen snapshots."), MDFNST_STRING, "" },
   { "path_sav", gettext_noop("Path override for save games and nonvolatile memory."), MDFNST_STRING, "" },
   { "path_state", gettext_noop("Path override for save states."), MDFNST_STRING, "" },
@@ -72,6 +76,21 @@ static MDFNSetting MednafenSettings[] =
   { "path_cheat", gettext_noop("Path override for cheats."), MDFNST_STRING, "" },
   { "path_palette", gettext_noop("Path override for custom palettes."), MDFNST_STRING, "" },
 
+  { "filesys.snap_samedir", gettext_noop("Write screen snapshots to the same directory the ROM/disk/disc image is in."),
+        MDFNST_BOOL, "0" },
+
+  { "filesys.sav_samedir", gettext_noop("Write/Read save games and nonvolatile memory to/from the same directory the ROM/disk/disc image is in."),
+	MDFNST_BOOL, "0" },
+
+  { "filesys.state_samedir", gettext_noop("Write/Read save states to/from the same directory the ROM/disk/disc image is in."),
+        MDFNST_BOOL, "0" },
+
+  { "filesys.movie_samedir", gettext_noop("Write/Read movies to/from the same directory the ROM/disk/disc image is in."),
+        MDFNST_BOOL, "0" },
+
+  { "filesys.disablesavegz", gettext_noop("Disable gzip compression when saving save states and backup memory."), MDFNST_BOOL, "0" },
+
+
   { "gb.vblur", CSD_vblur, MDFNST_BOOL, "0" },
   { "gb.vblur.accum", CSD_vblur_accum, MDFNST_BOOL, "0" },
   { "gb.vblur.accum.amount", CSD_vblur_accum_amount, MDFNST_FLOAT, "50", "0", "100" },
@@ -80,6 +99,10 @@ static MDFNSetting MednafenSettings[] =
   { "gba.vblur.accum", CSD_vblur_accum, MDFNST_BOOL, "0" },
   { "gba.vblur.accum.amount", CSD_vblur_accum_amount, MDFNST_FLOAT, "50", "0", "100" },
 
+  { "gg.vblur", CSD_vblur, MDFNST_BOOL, "0" },
+  { "gg.vblur.accum", CSD_vblur_accum, MDFNST_BOOL, "0" },
+  { "gg.vblur.accum.amount", CSD_vblur_accum_amount, MDFNST_FLOAT, "50", "0", "100" },
+
   { "lynx.vblur", CSD_vblur, MDFNST_BOOL, "0" },
   { "lynx.vblur.accum", CSD_vblur_accum, MDFNST_BOOL, "0" },
   { "lynx.vblur.accum.amount", CSD_vblur_accum_amount, MDFNST_FLOAT, "50", "0", "100" },
@@ -100,6 +123,10 @@ static MDFNSetting MednafenSettings[] =
   { "pcfx.vblur.accum", CSD_vblur_accum, MDFNST_BOOL, "0" },
   { "pcfx.vblur.accum.amount", CSD_vblur_accum_amount, MDFNST_FLOAT, "50", "0", "100" },
 
+  { "sms.vblur", CSD_vblur, MDFNST_BOOL, "0" },
+  { "sms.vblur.accum", CSD_vblur_accum, MDFNST_BOOL, "0" },
+  { "sms.vblur.accum.amount", CSD_vblur_accum_amount, MDFNST_FLOAT, "50", "0", "100" },
+
   { "wswan.vblur", CSD_vblur, MDFNST_BOOL, "0" },
   { "wswan.vblur.accum", CSD_vblur_accum, MDFNST_BOOL, "0" },
   { "wswan.vblur.accum.amount", CSD_vblur_accum_amount, MDFNST_FLOAT, "50", "0", "100" },
@@ -107,36 +134,6 @@ static MDFNSetting MednafenSettings[] =
   { NULL }
 };
 
-typedef struct
-{
- uint16 a, b, c, d;
-} HQPixelEntry;
-
-static uint32 *BlurBuf = NULL;
-static uint32 AccumBlurAmount; // max of 16384, infinite blur!
-static HQPixelEntry *AccumBlurBuf = NULL;
-
-static void InitVBlur(void)
-{
-	std::string sn = MDFNGameInfo->shortname;
-
-        if(MDFN_GetSettingB(std::string(sn + "." + std::string("vblur")).c_str()))
-        {
-         AccumBlurAmount = (uint32)(16384 * MDFN_GetSettingF(std::string(sn + "." + std::string("vblur.accum.amount")).c_str()) / 100);
-         if(MDFN_GetSettingB(std::string(sn + "." + std::string("vblur.accum")).c_str()))
-         {
-          AccumBlurBuf = (HQPixelEntry *)calloc(sizeof(HQPixelEntry), (MDFNGameInfo->pitch >> 2) * 256);
-          MDFN_printf(_("Video frame blur enabled with accumulation: %f.\n"), (double)AccumBlurAmount * 100 / 16384);
-         }
-         else
-         {
-          BlurBuf = (uint32 *)calloc(4, (MDFNGameInfo->pitch >> 2) * 256);
-          MDFN_printf(_("Video frame blur enabled.\n"));
-         }
-        }
-}
-
-
 static char *PortDeviceCache[16];
 static void *PortDataCache[16];
 static uint32 PortDataLenCache[16];
@@ -172,30 +169,55 @@ void MDFNI_CloseGame(void)
   }
   #endif
  }
- if(BlurBuf)
- {
-  free(BlurBuf);
-  BlurBuf = NULL;
- }
- if(AccumBlurBuf)
- {
-  free(AccumBlurBuf);
-  AccumBlurBuf = NULL;
- }
+ VBlur_Kill();
+
+ #ifdef WANT_DEBUGGER
+ MDFNDBG_Kill();
+ #endif
 }
 
 int MDFNI_NetplayStart(uint32 local_players, uint32 netmerge, const std::string &nickname, const std::string &game_key, const std::string &connect_password)
 {
  return(NetplayStart((const char**)PortDeviceCache, PortDataLenCache, local_players, netmerge, nickname, game_key, connect_password));
-
-
 }
 
 
+#ifdef WANT_NES_EMU
+extern MDFNGI EmulatedNES;
+#endif
 
-extern MDFNGI EmulatedNES, EmulatedGBA, EmulatedGB, EmulatedLynx, EmulatedNGP, EmulatedPCE, EmulatedPCFX, EmulatedWSwan;
+#ifdef WANT_GBA_EMU
+extern MDFNGI EmulatedGBA;
+#endif
+
+#ifdef WANT_GB_EMU
+extern MDFNGI EmulatedGB;
+#endif
+
+#ifdef WANT_LYNX_EMU
+extern MDFNGI EmulatedLynx;
+#endif
+
+#ifdef WANT_NGP_EMU
+extern MDFNGI EmulatedNGP;
+#endif
+
+#ifdef WANT_PCE_EMU
+extern MDFNGI EmulatedPCE;
+#endif
+
+#ifdef WANT_PCFX_EMU
+extern MDFNGI EmulatedPCFX;
+#endif
+
+#ifdef WANT_WSWAN_EMU
+extern MDFNGI EmulatedWSwan;
+#endif
+
+#ifdef WANT_SMS_EMU
+extern MDFNGI EmulatedSMS, EmulatedGG;
+#endif
 
-// The order of systems in this struct MUST match the GISYS_* defines in git.h! (FIXME so this is no longer required)
 MDFNGI *MDFNSystems[] = 
 { 
  #ifdef WANT_NES_EMU
@@ -229,6 +251,11 @@ MDFNGI *MDFNSystems[] =
  #ifdef WANT_WSWAN_EMU
  &EmulatedWSwan,
  #endif
+
+ #ifdef WANT_SMS_EMU
+ &EmulatedSMS,
+ &EmulatedGG,
+ #endif
 };
 
 unsigned int MDFNSystemCount = sizeof(MDFNSystems) / sizeof(MDFNGI *);
@@ -260,7 +287,7 @@ MDFNGI *MDFNI_LoadCD(const char *sysname, const char *devicename)
    CDIF_Track_Format format;
    if(CDIF_GetTrackFormat(track, format) && format == CDIF_FORMAT_MODE1)
    {
-    CDIF_ReadSector(sector_buffer, NULL, NULL, CDIF_GetTrackStartPositionLBA(track), 1);
+    CDIF_ReadSector(sector_buffer, NULL, CDIF_GetTrackStartPositionLBA(track), 1);
     if(!strncmp("PC-FX:Hu_CD-ROM", (char*)sector_buffer, strlen("PC-FX:Hu_CD-ROM")))
     {
      sysname = "pcfx";
@@ -290,12 +317,17 @@ MDFNGI *MDFNI_LoadCD(const char *sysname, const char *devicename)
    }
    CDInUse = 1;
 
+   #ifdef WANT_DEBUGGER
+   MDFNDBG_PostGameLoad(); 
+   #endif
+
    MDFNSS_CheckStates();
    MDFNMOV_CheckMovies();
 
    MDFN_ResetMessages();   // Save state, status messages, etc.
-   //MDFN_indent(-1);
-   InitVBlur();
+
+   VBlur_Init();
+
    MDFN_StateEvilBegin();
    return(MDFNGameInfo);
   }
@@ -311,7 +343,7 @@ MDFNGI *MDFNI_LoadGame(const char *name)
 {
         MDFNFILE *fp;
 	struct stat stat_buf;
-	static const char *valid_iae =	".nes\0.fds\0.nsf\0.nsfe\0.unf\0.unif\0.nez\0.gb\0.gbc\0.gba\0.agb\0.cgb\0.bin\0.pce\0.hes\0.ngp\0.ngc\0.ws\0.wsc\0";
+	static const char *valid_iae =	".nes\0.fds\0.nsf\0.nsfe\0.unf\0.unif\0.nez\0.gb\0.gbc\0.gba\0.agb\0.cgb\0.bin\0.pce\0.hes\0.sgx\0.ngp\0.ngc\0.ws\0.wsc\0";
 
 	#ifdef NEED_CDEMU
 	if(strlen(name) > 4 && (!strcasecmp(name + strlen(name) - 4, ".cue") || !strcasecmp(name + strlen(name) - 4, ".toc")))
@@ -338,10 +370,8 @@ MDFNGI *MDFNI_LoadGame(const char *name)
         GetFileBase(name);
 
 	fp=MDFN_fopen(name, MDFN_MakeFName(MDFNMKF_IPS,0,0).c_str(),"rb", valid_iae);
-
 	if(!fp)
         {
- 	 MDFN_PrintError(_("Error opening \"%s\": %s"), name, strerror(errno));
 	 MDFNGameInfo = NULL;
 	 return 0;
 	}
@@ -354,8 +384,9 @@ MDFNGI *MDFNI_LoadGame(const char *name)
          MDFNGameInfo = MDFNSystems[x];
          MDFNGameInfo->soundchan = 0;
          MDFNGameInfo->soundrate = 0;
-         MDFNGameInfo->name=0;
+         MDFNGameInfo->name = NULL;
          MDFNGameInfo->rotated = 0;
+
 	 t = MDFNGameInfo->Load(name, fp);
 
 	 if(t == 0)
@@ -382,6 +413,10 @@ MDFNGI *MDFNI_LoadGame(const char *name)
 
         MDFN_fclose(fp);
 
+	#ifdef WANT_DEBUGGER
+	MDFNDBG_PostGameLoad();
+	#endif
+
 	MDFNSS_CheckStates();
 	MDFNMOV_CheckMovies();
 
@@ -405,7 +440,7 @@ MDFNGI *MDFNI_LoadGame(const char *name)
           *tmp = 0;
         }
 
-	InitVBlur();
+	VBlur_Init();
 
         MDFN_StateEvilBegin();
         return(MDFNGameInfo);
@@ -448,7 +483,12 @@ int MDFNI_Initialize(char *basedir, const std::vector<MDFNSetting> &DriverSettin
 
         if(!MFDN_LoadSettings(basedir))
 	 return(0);
-        return 1;
+
+	#ifdef WANT_DEBUGGER
+	MDFNDBG_Init();
+	#endif
+
+        return(1);
 }
 
 void MDFNI_Kill(void)
@@ -466,16 +506,6 @@ void MDFNI_Kill(void)
  }
 }
 
-void MDFNI_DoRewind(void)
-{
- if(MDFNMOV_IsPlaying())
-  MDFN_DispMessage((UTF8*)_("Can't rewind during movie playback(yet!)."));
- else if(MDFNnetplay)
-  MDFN_DispMessage((UTF8*)_("Silly-billy, can't rewind during netplay."));
- else
-  MDFNGameInfo->DoRewind();
-}
-
 static uint8 *gbpad;
 
 void MDFNI_SetGBInput(uint8 *yaya)
@@ -485,14 +515,14 @@ void MDFNI_SetGBInput(uint8 *yaya)
 
 int doothack = -1;
 
-void MDFNI_Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int32 *SoundBufSize, int skip, float soundmultiplier)
+void MDFNI_Emulate(EmulateSpecStruct *espec) //uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int32 *SoundBufSize, int skip, float soundmultiplier)
 {
- FSettings.soundmultiplier = soundmultiplier;
+ FSettings.soundmultiplier = espec->soundmultiplier;
 
- if(soundmultiplier != LastSoundMultiplier)
+ if(espec->soundmultiplier != LastSoundMultiplier)
  {
-  MDFNGameInfo->SetSoundMultiplier(soundmultiplier);
-  LastSoundMultiplier = soundmultiplier;
+  MDFNGameInfo->SetSoundMultiplier(espec->soundmultiplier);
+  LastSoundMultiplier = espec->soundmultiplier;
  }
 
  #ifdef NETWORK
@@ -506,98 +536,94 @@ void MDFNI_Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int32
   if(PortDataCache[x])
    MDFNMOV_AddJoy(PortDataCache[x], PortDataLenCache[x]);
 
- if(BlurBuf || AccumBlurBuf)
-  skip = 0;
+ if(VBlur_IsOn())
+  espec->skip = 0;
 
- MDFNGameInfo->Emulate(pXBuf, LineWidths, SoundBuf, SoundBufSize, skip);
-
- if(AccumBlurBuf)
+ if(espec->NeedRewind)
  {
-  uint32 pitchp = MDFNGameInfo->pitch >> 2;
-  for(int y = 0; y < MDFNGameInfo->DisplayRect.h; y++)
+  if(MDFNMOV_IsPlaying())
   {
-   int xw = MDFNGameInfo->DisplayRect.w;
-   int xs = MDFNGameInfo->DisplayRect.x;
-
-   if(LineWidths[0].w != ~0)
-    xw = LineWidths[MDFNGameInfo->DisplayRect.y + y].w;
-
-   if(AccumBlurAmount == 8192)
-   {
-    for(int x = 0; x < xw; x++)
-    {
-     uint32 color;
-     HQPixelEntry mixcolor;
-     color = pXBuf[(y + MDFNGameInfo->DisplayRect.y) * pitchp + xs + x];
-     mixcolor = AccumBlurBuf[y * pitchp + x];
-     mixcolor.a = ((uint32)mixcolor.a + ((color & 0xFF) << 8)) >> 1;
-     mixcolor.b = ((uint32)mixcolor.b + ((color & 0xFF00))) >> 1;
-     mixcolor.c = ((uint32)mixcolor.c + ((color & 0xFF0000) >> 8)) >> 1;
-     mixcolor.d = ((uint32)mixcolor.d + ((color & 0xFF000000) >> 16)) >> 1;
-    AccumBlurBuf[y * pitchp + x] = mixcolor;
-    pXBuf[(y + MDFNGameInfo->DisplayRect.y) * pitchp + xs + x] = ((mixcolor.a >> 8) << 0) | ((mixcolor.b >> 8) << 8) | 
-	((mixcolor.c >> 8) << 16) | ((mixcolor.d >> 8) << 24);
-    }
-   }
-   else
-   {
-    uint32 InvAccumBlurAmount = 16384 - AccumBlurAmount;
-
-    for(int x = 0; x < xw; x++)
-    {
-     uint32 color;
-     HQPixelEntry mixcolor;
-     color = pXBuf[(y + MDFNGameInfo->DisplayRect.y) * pitchp + xs + x];
-     mixcolor = AccumBlurBuf[y * pitchp + x];
-     mixcolor.a = ((uint32)mixcolor.a * AccumBlurAmount + InvAccumBlurAmount * ((color & 0xFF) << 8)) >> 14;
-     mixcolor.b = ((uint32)mixcolor.b * AccumBlurAmount + InvAccumBlurAmount * ((color & 0xFF00))) >> 14;
-     mixcolor.c = ((uint32)mixcolor.c * AccumBlurAmount + InvAccumBlurAmount * ((color & 0xFF0000) >> 8)) >> 14;
-     mixcolor.d = ((uint32)mixcolor.d * AccumBlurAmount + InvAccumBlurAmount * ((color & 0xFF000000) >> 16)) >> 14;
-    AccumBlurBuf[y * pitchp + x] = mixcolor;
-    pXBuf[(y + MDFNGameInfo->DisplayRect.y) * pitchp + xs + x] = ((mixcolor.a >> 8) << 0) | ((mixcolor.b >> 8) << 8) |
-        ((mixcolor.c >> 8) << 16) | ((mixcolor.d >> 8) << 24);
-    }
-   }
-
+   espec->NeedRewind = 0;
+   MDFN_DispMessage((UTF8*)_("Can't rewind during movie playback(yet!)."));
   }
- }
- else if(BlurBuf)
- {
-  uint32 pitchp = MDFNGameInfo->pitch >> 2;
-  for(int y = 0; y < MDFNGameInfo->DisplayRect.h; y++)
+  else if(MDFNnetplay)
   {
-   int xw = MDFNGameInfo->DisplayRect.w;
-   int xs = MDFNGameInfo->DisplayRect.x;
-
-   if(LineWidths[0].w != ~0)
-    xw = LineWidths[MDFNGameInfo->DisplayRect.y + y].w;
-
-   for(int x = 0; x < xw; x++)
-   {
-    uint32 color, mixcolor;
-    color = pXBuf[(y + MDFNGameInfo->DisplayRect.y) * pitchp + xs + x];
-    mixcolor = BlurBuf[y * pitchp + x];
-    BlurBuf[y * pitchp + x] = color;
-    color = (((color & 0xFF) + (mixcolor & 0xFF)) >> 1) | ((((color & 0xFF00) + (mixcolor & 0xFF00)) >> 1) & 0xFF00) |
-       ((((color & 0xFF0000) + (mixcolor & 0xFF0000)) >> 1) & 0xFF0000) | ((((color >> 24) + (mixcolor >> 24)) >> 1) << 24);
-    pXBuf[(y + MDFNGameInfo->DisplayRect.y) * pitchp + xs + x] = color;
-   }
+   espec->NeedRewind = 0;
+   MDFN_DispMessage((UTF8*)_("Silly-billy, can't rewind during netplay."));
+  }
+  else if(MDFNGameInfo->GameType == GMT_PLAYER)
+  {
+   espec->NeedRewind = 0;
+   MDFN_DispMessage((UTF8*)_("Music player rewinding is unsupported."));
   }
  }
 
- if(SoundBuf && SoundBufSize)
+ espec->NeedSoundReverse = MDFN_StateEvil(espec->NeedRewind);
+
+ MDFNGameInfo->Emulate(espec);
+
+ VBlur_Run(espec);
+
+ if(espec->SoundBuf && espec->SoundBufSize)
  {
-  //if(CDInUse)
-  // CDIF_Sound_Mix(*SoundBuf, *SoundBufSize);
-  MDFN_WriteWaveData(*SoundBuf, *SoundBufSize); /* This function will just return if sound recording is off. */
+  if(espec->NeedSoundReverse)
+  {
+   int16 *yaybuf = *(espec->SoundBuf);
+   int32 slen = *(espec->SoundBufSize);
+
+   if(MDFNGameInfo->soundchan == 1)
+   {
+    for(int x = 0; x < (slen / 2); x++)    
+    {
+     int16 cha = yaybuf[slen - x - 1];
+     yaybuf[slen - x - 1] = yaybuf[x];
+     yaybuf[x] = cha;
+    }
+   }
+   else if(MDFNGameInfo->soundchan == 2)
+   {
+    for(int x = 0; x < (slen * 2) / 2; x++)
+    {
+     int16 cha = yaybuf[slen * 2 - (x&~1) - ((x&1) ^ 1) - 1];
+     yaybuf[slen * 2 - (x&~1) - ((x&1) ^ 1) - 1] = yaybuf[x];
+     yaybuf[x] = cha;
+    }
+   }
+  }
+  MDFN_WriteWaveData(*(espec->SoundBuf), *(espec->SoundBufSize)); /* This function will just return if sound recording is off. */
+ }
+}
+
+// This function should only be called for state rewinding.
+// FIXME:  Add a macro for SFORMAT structure access instead of direct access
+int MDFN_RawInputStateAction(StateMem *sm, int load, int data_only)
+{
+ static const char *stringies[16] = { "RI00", "RI01", "RI02", "RI03", "RI04", "RI05", "RI06", "RI07", "RI08", "RI09", "RI0a", "RI0b", "RI0c", "RI0d", "RI0e", "RI0f" };
+ SFORMAT StateRegs[17];
+ int x;
+
+ for(x = 0; x < 16; x++)
+ {
+  StateRegs[x].desc = stringies[x];
+  if(PortDataCache[x])
+  {
+   StateRegs[x].v = PortDataCache[x];
+   StateRegs[x].s = PortDataLenCache[x];
+  }
+  else
+  {
+   StateRegs[x].v = NULL;
+   StateRegs[x].s = 0;
+  }
  }
 
- //if(doothack >= 0)
- // if(--doothack)
- // {
- //  MDFN_SavePNGSnapshot("test.png", pXBuf, LineWidths, 1024 * 4);
- //  exit(1);
- // }
+ StateRegs[x].v = NULL;
+ StateRegs[x].s = 0;
+ StateRegs[x].desc = NULL;
+
+ int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "rinp");
+
+ return(ret);
 }
 
 
