@@ -1,8 +1,8 @@
 #ifndef _MEDNAFEN_H
 
-#include <stdio.h>
-
 #include "types.h"
+#include <stdio.h>
+#include <stdlib.h>
 #include "gettext.h"
 
 #define _(String) gettext (String)
@@ -46,6 +46,9 @@ void MDFN_FlushGameCheats(int nosave);
 void MDFN_DoSimpleCommand(int cmd);
 void MDFN_QSimpleCommand(int cmd);
 
+#include "state.h"
+int MDFN_RawInputStateAction(StateMem *sm, int load, int data_only);
+
 #include "mednafen-driver.h"
 
 #define _MEDNAFEN_H
