@@ -228,7 +228,7 @@ int UNROM_Init(CartInfo *info)
  info->StateAction = UNROM_StateAction;
 
  #ifdef WANT_DEBUGGER
- MDFNNES_AddRegGroup(&DBGUNROMRegsGroup);
+ MDFNDBG_AddRegGroup(&DBGUNROMRegsGroup);
  #endif
 
  //PRGmask16[0]&=7;
