@@ -19,8 +19,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <string>
 #include <zlib.h>
@@ -207,7 +205,7 @@ uint32 iNESGameCRC32;
 
 struct CRCMATCH	{
 	uint32 crc;
-	char *name;
+	const char *name;
 };
 
 struct INPSEL {
@@ -310,7 +308,7 @@ static void SetInput(void)
 
 struct BADINF {
         uint64 md5partial;
-        char *name;
+        const char *name;
         uint32 type;
 };
 
@@ -500,7 +498,7 @@ static void CheckHInfo(void)
 
   if(tofix&2)
   {
-   char *mstr[3]={"Mirroring should be set to \"Horizontal\".  ",
+   const char *mstr[3]={"Mirroring should be set to \"Horizontal\".  ",
 		  "Mirroring should be set to \"Vertical\".  ",
 		  "Mirroring should be set to \"Four-screen\".  "
 		 };
@@ -511,7 +509,7 @@ static void CheckHInfo(void)
    MDFN_printf(_("The battery-backed bit should be set.\n"));
   }
   if(tofix&8)
-	  {
+  {
    MDFN_printf(_("This game should not have any CHR ROM.  "));
   }
   MDFN_indent(-1);
@@ -525,19 +523,27 @@ typedef struct {
 	void (*init)(CartInfo *);
 } NewMI;
 
-int iNESLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
+bool iNES_TestMagic(const char *name, MDFNFILE *fp)
+{
+ if(fp->size < 16)
+  return(FALSE);
+
+ if(memcmp(fp->data, "NES\x1a", 4))
+  return(FALSE);
+
+ return(TRUE);
+}
+
+bool iNESLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 {
         md5_context md5;
 
-	/* File size is too small to be an iNES file */
-	if(MDFN_fread(&head, 1, 16,fp) != 16)
-	 return(-1);
-	
-	memcpy(&head, fp->data, 16);
+	if(!iNES_TestMagic(name, fp))
+	 return(FALSE);
 
-	/* Is the magic right? */
-        if(memcmp(&head,"NES\x1a",4))
-         return(-1);
+	/* File size is too small to be an iNES file */
+        memcpy(&head, fp->data, 16);
+	MDFN_fseek(fp, 16, SEEK_SET);
 
 	memset(&iNESCart,0,sizeof(iNESCart));
 
@@ -588,7 +594,7 @@ int iNESLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 	}
 
 	#ifdef WANT_DEBUGGER
-	MDFNNES_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "prgrom", "PRG ROM", (unsigned int)(log(ROM_size << 14) / log(2)));
+	MDFNDBG_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "prgrom", "PRG ROM", (unsigned int)(log(ROM_size << 14) / log(2)));
 	#endif
 
         if (VROM_size) 
@@ -599,7 +605,7 @@ int iNESLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 	  return 0;
 	 }
 	 #ifdef WANT_DEBUGGER
-	 MDFNNES_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "chrrom", "CHR ROM", (unsigned int)(log(VROM_size << 13) / log(2)));
+	 MDFNDBG_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "chrrom", "CHR ROM", (unsigned int)(log(VROM_size << 13) / log(2)));
 	 #endif
 	}
         memset(ROM,0xFF,ROM_size<<14);
@@ -864,7 +870,7 @@ static int NewiNES_Init(int num)
     SetupCartCHRMapping(0x0,VROM,CHRRAMSize,1);
 
     #ifdef WANT_DEBUGGER
-    MDFNNES_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "chrram", "CHR RAM", (unsigned int)(log(CHRRAMSize) / log(2)));
+    MDFNDBG_AddASpace(iNES_GetAddressSpaceBytes, iNES_PutAddressSpaceBytes, "chrram", "CHR RAM", (unsigned int)(log(CHRRAMSize) / log(2)));
     #endif
    }
    tmp->init(&iNESCart);
