@@ -24,7 +24,9 @@
 
 X6502 X;
 
+#ifdef WANT_DEBUGGER
 void (*X6502_Run)(int32 cycles);
+#endif
 
 uint32 timestamp;
 void (*MapIRQHook)(int a);
@@ -62,6 +64,7 @@ static INLINE void WrMemNorm(unsigned int A, uint8 V)
  BWrite[A](A,V);
 }
 
+#ifdef WANT_DEBUGGER
 X6502 XSave;     /* This is getting ugly. */
 //#define RdMemHook(A)	( X.ReadHook?(_DB=X.ReadHook(&X,A)):(_DB=ARead[A](A)) )
 //#define WrMemHook(A,V)	{ if(X.WriteHook) X.WriteHook(&X,A,V); else BWrite[A](A,V); }
@@ -81,6 +84,7 @@ static INLINE void WrMemHook(unsigned int A, uint8 V)
  else if(!X.preexec)
   BWrite[A](A,V);
 }
+#endif
 
 uint8 X6502_DMR(uint32 A)
 {
@@ -353,7 +357,7 @@ static uint8 CycTable[256] =
 /*0xB0*/ 2,5,2,5,4,4,4,4,2,4,2,4,4,4,4,4,
 /*0xC0*/ 2,6,2,8,3,3,5,5,2,2,2,2,4,4,6,6,
 /*0xD0*/ 2,5,2,8,4,4,6,6,2,4,2,7,4,4,7,7,
-/*0xE0*/ 2,6,3,8,3,3,5,5,2,2,2,2,4,4,6,6,
+/*0xE0*/ 2,6,2,8,3,3,5,5,2,2,2,2,4,4,6,6,
 /*0xF0*/ 2,5,2,8,4,4,6,6,2,4,2,7,4,4,7,7,
 };
 
@@ -416,7 +420,9 @@ void X6502_Init(void)
 	 if(!x) ZNTable[x]=Z_FLAG;
 	 else if (x&0x80) ZNTable[x]=N_FLAG;
 	 else ZNTable[x]=0;
+	#ifdef WANT_DEBUGGER
 	X6502_Debug(0,0,0);
+	#endif
 }
 
 void X6502_Power(void)
@@ -426,6 +432,7 @@ void X6502_Power(void)
  X6502_Reset();
 }
 
+#ifdef WANT_DEBUGGER
 static void X6502_RunDebug(int32 cycles)
 {
 	#define RdMem RdMemHook
@@ -553,9 +560,13 @@ static void X6502_RunDebug(int32 cycles)
 
 static void X6502_RunNormal(int32 cycles)
 {
+#else
+void X6502_Run(int32 cycles)
+{
+#endif
 	#define RdMem RdMemNorm
 	#define WrMem WrMemNorm
-        #define ADDBT(x) NESDBG_AddBranchTrace(x)
+        #define ADDBT(x)
 
         uint32 pbackus;
 
@@ -694,6 +705,7 @@ int X6502_StateAction(StateMem *sm, int load, int data_only)
  return(MDFNSS_StateAction(sm, load, data_only, love));
 }
 
+#ifdef WANT_DEBUGGER
 void X6502_Debug(void (*CPUHook)(uint32),
 		uint8 (*ReadHook)(X6502 *, unsigned int),
 		void (*WriteHook)(X6502 *, unsigned int, uint8))
@@ -708,3 +720,4 @@ void X6502_Debug(void (*CPUHook)(uint32),
  else
   X6502_Run=X6502_RunDebug;
 }
+#endif
