@@ -66,13 +66,13 @@
 namespace TLCS900H
 {
 
-static void DUMMY_instruction_error(char* vaMessage,...)
+static void DUMMY_instruction_error(const char* vaMessage,...)
 {
 
 
 }
 
-void (*instruction_error)(char* vaMessage,...) = DUMMY_instruction_error;
+void (*instruction_error)(const char* vaMessage,...) = DUMMY_instruction_error;
 
 //=========================================================================
 
