@@ -19,12 +19,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <stdio.h>		// debug
 #include "pce.h"
 #include "psg.h"
 #include "cdrom.h"
 #include "adpcm.h"
-#include "../scsicd.h"
 
 #define ADPCM_MAXVOLUME			1024
 
@@ -266,17 +264,6 @@ bool ADPCM_Init()
 	 return(0);
 	}
 
-	CurrentVolume = InitialVolume;
-	VolumeStep = InitialVolume / 10;
-	bFadeIn  = FALSE;
-	bFadeOut = FALSE;
-	ClockCount = 0;
-	FadeCycle = 0;
-	LastCmd = 0;
-
-	ad_sample = 2048;
-	ad_ref_index = 0;
-
 	ADPCMVolumeSetting = MDFN_GetSettingUI("pce.adpcmvolume");
 	if(ADPCMVolumeSetting != 100)
 	{
@@ -288,7 +275,7 @@ bool ADPCM_Init()
 	bigdivacc = (int32)((double)1789772.727272 * 4 * 65536 / 32087.5 * pce_overclocked);
 
 	#ifdef WANT_DEBUGGER
-	PCE_AddASpace(ADPCM_GetAddressSpaceBytes, ADPCM_PutAddressSpaceBytes, "adpcm", "ADPCM RAM", 16);
+	MDFNDBG_AddASpace(ADPCM_GetAddressSpaceBytes, ADPCM_PutAddressSpaceBytes, "adpcm", "ADPCM RAM", 16);
 	#endif
 
 	return TRUE;
@@ -299,16 +286,12 @@ void ADPCM_Close()
 {
 	if(RAM)
 	{
-	 free(RAM);
+	 MDFN_free(RAM);
 	 RAM = NULL;
 	}
 }
 
 
-/*-----------------------------------------------------------------------------
-	[Reset]
-		
------------------------------------------------------------------------------*/
 void ADPCM_Reset()
 {
 	_pfnNotification(ADPCM_STATE_NORMAL);
@@ -336,6 +319,26 @@ void ADPCM_Reset()
 	RedoLPF(SampleFreq);
 }
 
+void ADPCM_Power(void)
+{
+ memset(RAM, 0x00, 65536);
+
+ ReadPending = WritePending = 0;
+ ReadBuffer = 0;
+
+ CurrentVolume = InitialVolume;
+ VolumeStep = InitialVolume / 10;
+ bFadeIn  = FALSE;
+ bFadeOut = FALSE;
+ ClockCount = 0;
+ FadeCycle = 0;
+ LastCmd = 0;
+
+ ad_sample = 2048;
+ ad_ref_index = 0;
+
+ ADPCM_Reset();
+}
 
 /*-----------------------------------------------------------------------------
 	[SetNotificationFunction]
