@@ -25,6 +25,12 @@ of samples between the ~60Hz peaks(just to verify that the math shown below is r
 The spectrum peaked at 15734 Hz.  21477272.727272... / 3 / 15734 = 455.00(CPU cycles per scanline)"
 */
 
+#define VDC_DEBUG(x) 
+//printf("%s: %d\n", x, vdc->display_counter);
+#define VDC_UNDEFINED(x) { }
+//{ printf("%s: %d\n", x, vdc->display_counter); }
+
+
 #include <math.h>
 #include "pce.h"
 #include "../video.h"
@@ -40,11 +46,21 @@ static uint32 amask_shift;
 static uint32 userle; // User layer enable.
 static bool unlimited_sprites;
 
+static const int VRAM_Size = 0x8000;
+static const int VRAM_SizeMask = VRAM_Size - 1; //0x7FFF;
+static const int VRAM_BGTileNoMask = VRAM_SizeMask / 16; //0x7FF;
+
 #define ULE_BG0		1
 #define ULE_SPR0	2
 #define ULE_BG1		4
 #define ULE_SPR1	8
 
+static const unsigned int bat_width_tab[4] = { 32, 64, 128, 128 };
+static const unsigned int bat_width_shift_tab[4] = { 5, 6, 7, 7 };
+static const unsigned int bat_height_tab[2] = { 32, 64 };
+static const unsigned int ClockModeWidths[3] = { 288, 384, 576 };
+static const unsigned int ClockPixelWidths[3] = { 341, 455, 682 };
+
 static unsigned int VDS;
 static unsigned int VSW;
 static unsigned int VDW;
@@ -76,6 +92,11 @@ typedef struct
         uint8 select;
         uint16 MAWR;    // Memory Address Write Register
         uint16 MARR;    // Memory Address Read Register
+
+	#ifdef WANT_DEBUGGER
+	uint16 MAWR_BP, MARR_BP; // Special variables so VRAM breakpoints will work correctly when block transfer instructions are used.
+	#endif
+
         uint16 CR;      // Control Register
         uint16 RCR;     // Raster Compare Register
         uint16 BXR;     // Background X-Scroll Register
@@ -103,8 +124,8 @@ typedef struct
         uint8 status;
 
         uint16 SAT[0x100];
-        uint16 VRAM[32768];
-        uint8 bg_tile_cache[2048][8][8] __attribute__ ((aligned (8))); // Tile, y, x
+        uint16 VRAM[VRAM_Size];
+        uint8 bg_tile_cache[VRAM_Size / 16][8][8] __attribute__ ((aligned (8))); // Tile, y, x
 
         uint16 DMAReadBuffer;
         bool8 DMAReadWrite;
@@ -195,12 +216,15 @@ uint32 VDC_GetRegister(const std::string &oname, std::string *special)
  char chip_c = 0;
  int chip = 0;
  std::string name;
+ vdc_t *vdc;
 
  trio_sscanf(oname.c_str(), "%63[^-]%*[-]%c", neoname, &chip_c);
 
  if(chip_c == 'B')
   chip = 1;
 
+ vdc = vdc_chips[chip];
+
  name = std::string(neoname);
 
  if(name == "MAWR")
@@ -239,7 +263,18 @@ uint32 VDC_GetRegister(const std::string &oname, std::string *special)
  else if(name == "VCR")
   return(vdc_chips[chip]->VCR);
  else if(name == "DCR")
+ {
+  if(special)
+  {
+   char buf[256];
+   snprintf(buf, 256, "SATB DMA IRQ: %s, VRAM DMA IRQ: %s, DMA Source Address: %s, DMA Dest Address: %s, Auto SATB DMA: %s",
+	(vdc->DCR & 0x1) ? "On" : "Off", (vdc->DCR & 0x2) ? "On" : "Off", (vdc->DCR & 0x4) ? "Decrement" : "Increment", (vdc->DCR & 0x8) ? "Decrement" : "Increment", 
+	(vdc->DCR & 0x10) ? "On" : "Off");
+
+   *special = std::string(buf);
+  }
   return(vdc_chips[chip]->DCR);
+ }
  else if(name == "SOUR")
   return(vdc_chips[chip]->SOUR);
  else if(name == "DESR")
@@ -354,7 +389,7 @@ void VDC_GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, u
  {
   while(Length--)
   {
-   Address &= 0xFFFF;
+   Address &= (VRAM_Size << 1) - 1;
    *Buffer = vdc_chips[chip]->VRAM[Address >> 1] >> ((Address & 1) * 8);
    Address++;
    Buffer++;
@@ -381,7 +416,7 @@ void VDC_PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, u
  {
   while(Length--)
   {
-   Address &= 0xFFFF;
+   Address &= (VRAM_Size << 1) - 1;
    REGSETP(vdc_chips[chip]->VRAM[Address >> 1], *Buffer, Address & 1);
    FixTileCache(vdc_chips[chip], Address >> 1);
    Address++;
@@ -498,6 +533,16 @@ bool VDC_ToggleLayer(int which)
 }
 
 #ifdef WANT_DEBUGGER 
+
+void VDC_BP_Start(void)
+{
+ for(int chip = 0; chip < TotalChips; chip++)
+ {
+  vdc_chips[chip]->MAWR_BP = vdc_chips[chip]->MAWR;
+  vdc_chips[chip]->MARR_BP = vdc_chips[chip]->MARR;
+ }
+}
+
 DECLFR(VDC_Read_TestBP)
 {
  int msb = A & 1;
@@ -524,7 +569,10 @@ DECLFR(VDC_Read_TestBP)
   case 0x3:
            if(vdc->select == 0x2) // VRR - VRAM Read Register
             if(msb)
-             PCEDBG_CheckBP(BPOINT_AUX_READ, ((vdc->MARR + vram_inc_tab[(vdc->CR >> 11) & 0x3]) & 0x7FFF) | (chip ? 0x10000 : 0x00000), 1);
+	    {
+             vdc->MARR_BP += vram_inc_tab[(vdc->CR >> 11) & 0x3];
+             PCEDBG_CheckBP(BPOINT_AUX_READ, vdc->MARR_BP | (chip ? 0x10000 : 0x00000), 1);
+	    }
            break;
  }
 
@@ -590,7 +638,11 @@ DECLFR(VDC_Read)
 	     if(!PCE_InDebug)
 	     {
 	      vdc->MARR += vram_inc_tab[(vdc->CR >> 11) & 0x3];
-	      vdc->read_buffer = vdc->VRAM[vdc->MARR & 0x7FFF];
+
+	      if(vdc->MARR >= VRAM_Size)
+	       VDC_UNDEFINED("Unmapped VRAM VRR read");
+
+	      vdc->read_buffer = vdc->VRAM[vdc->MARR & VRAM_SizeMask];
 	     }
 	    }
 	   }
@@ -634,18 +686,24 @@ static void DoDMA(vdc_t *vdc)
      {
       if(!vdc->DMAReadWrite)
       {
-       vdc->DMAReadBuffer = vdc->VRAM[vdc->SOUR & 0x7FFF];
+       if(vdc->SOUR >= VRAM_Size)
+        VDC_UNDEFINED("Unmapped VRAM DMA read");
+
+       vdc->DMAReadBuffer = vdc->VRAM[vdc->SOUR & VRAM_SizeMask];
       }
       else
       {
-       if(vdc->DESR < 0x8000)
+       if(vdc->DESR < VRAM_Size)
        {
         vdc->VRAM[vdc->DESR] = vdc->DMAReadBuffer;
         FixTileCache(vdc, vdc->DESR);
        }
 
-       vdc->DESR += (((vdc->DCR & 0x4) >> 1) ^ 2) - 1;
-       vdc->SOUR += (((vdc->DCR & 0x8) >> 2) ^ 2) - 1;
+       //if(vdc->DCR & 0xC) 
+	//printf("Pllal: %02x\n", vdc->DCR);
+
+       vdc->SOUR += (((vdc->DCR & 0x4) >> 1) ^ 2) - 1;
+       vdc->DESR += (((vdc->DCR & 0x8) >> 2) ^ 2) - 1;
        vdc->LENR--;
        if(vdc->LENR == 0xFFFF)  // DMA is done.
        {
@@ -654,7 +712,7 @@ static void DoDMA(vdc_t *vdc)
         {
          vdc->status |= VDCS_DV;
          HuC6280_IRQBegin(MDFN_IQIRQ1);
-	 //puts("DMA IRQ");
+	 VDC_DEBUG("DMA IRQ");
         }
         break;
        }
@@ -692,10 +750,13 @@ DECLFW(VDC_Write_TestBP)
            switch(vdc->select & 0x1F)
            {
             case 0x01: if(msb)
-                        PCEDBG_CheckBP(BPOINT_AUX_READ, (vdc->MARR & 0x00FF) | (V << 8) | (chip ? 0x10000 : 0x00000), 1);
+                        PCEDBG_CheckBP(BPOINT_AUX_READ, (vdc->MARR_BP & 0x00FF) | (V << 8) | (chip ? 0x10000 : 0x00000), 1);
                        break;
             case 0x02: if(msb)
-                        PCEDBG_CheckBP(BPOINT_AUX_WRITE, vdc->MAWR | (chip ? 0x10000 : 0x00000), 1);
+		       {
+                        PCEDBG_CheckBP(BPOINT_AUX_WRITE, vdc->MAWR_BP | (chip ? 0x10000 : 0x00000), 1);
+                        vdc->MAWR_BP += vram_inc_tab[(vdc->CR >> 11) & 0x3];
+		       }
                        break;
            }
  }
@@ -733,7 +794,10 @@ DECLFW(VDC_Write)
   vdc = vdc_chips[0];
   A &= 0x3;
  }
- //printf("%04x: %02x\n", A, V);
+ //if((A == 0x2 || A == 0x3) && ((vdc->select & 0x1f) >= 0x09) && ((vdc->select & 0x1f) <= 0x13))
+
+ //printf("%04x, %02x: %02x, %d\n", A, vdc->select, V, vdc->display_counter);
+
  switch(A)
  {
   case 0x0: vdc->select = V & 0x1F; break;
@@ -745,17 +809,25 @@ DECLFW(VDC_Write)
 	    case 0x01: REGSETP(vdc->MARR, V, msb);
 		       if(msb)
 		       {
+			if(vdc->MARR >= VRAM_Size)
+                         VDC_UNDEFINED("Unmapped VRAM VRR(MARR set) read");
+
 			vdc->read_buffer = vdc->VRAM[vdc->MARR];
 		       }
 	               break;
 	    case 0x02: if(!msb) vdc->write_latch = V;
 		       else
 		       {
-			if(vdc->MAWR < 0x8000)
+			if(vdc->MAWR < VRAM_Size)
 			{
- 			 vdc->VRAM[vdc->MAWR & 0x7fff] = (V << 8) | vdc->write_latch;
-			 FixTileCache(vdc, vdc->MAWR & 0x7FFF);
+ 			 vdc->VRAM[vdc->MAWR & VRAM_SizeMask] = (V << 8) | vdc->write_latch;
+			 FixTileCache(vdc, vdc->MAWR & VRAM_SizeMask);
 			} 
+			else
+			{
+	                 //VDC_UNDEFINED("Unmapped VRAM write");
+			 //printf("VROOM: %04x, %02x\n", vdc->MAWR, (vdc->CR >> 11) & 0x3);
+			}
 	                vdc->MAWR += vram_inc_tab[(vdc->CR >> 11) & 0x3];
 		       }
 		       break;
@@ -773,9 +845,9 @@ DECLFW(VDC_Write)
 	    case 0x0d: REGSETP(vdc->VDR, V, msb); break;
 	    case 0x0e: REGSETP(vdc->VCR, V, msb); break;
 	    case 0x0f: REGSETP(vdc->DCR, V, msb); break;
-	    case 0x10: REGSETP(vdc->SOUR, V, msb); /*printf("SOUR: %04x\n", vdc->SOUR); */ break;
-	    case 0x11: REGSETP(vdc->DESR, V, msb); /*printf("DESR: %04x\n", vdc->DESR); */ break;
-	    case 0x12: REGSETP(vdc->LENR, V, msb); /*printf("LENR: %04x, %d\n", vdc->LENR, HuCPU.timestamp); */
+	    case 0x10: REGSETP(vdc->SOUR, V, msb); break;
+	    case 0x11: REGSETP(vdc->DESR, V, msb); break;
+	    case 0x12: REGSETP(vdc->LENR, V, msb);
 		       if(msb)
 		       {
 			vdc->DMARunning = 1;
@@ -798,12 +870,6 @@ DECLFW(VDC_Write)
  }
 }
 
-static const unsigned int bat_width_tab[4] = { 32, 64, 128, 128 };
-static const unsigned int bat_width_shift_tab[4] = { 5, 6, 7, 7 };
-static const unsigned int bat_height_tab[2] = { 32, 64 };
-static const unsigned int ClockModeWidths[3] = { 288, 384, 576 };
-static const unsigned int ClockPixelWidths[3] = { 341, 455, 682 };
-
 static INLINE void CalcWidthStartEnd(vdc_t *vdc, uint32 &display_width, uint32 &start, uint32 &end)
 {
  display_width = (M_vdc_HDW + 1) * 8;
@@ -844,8 +910,10 @@ static void DrawBG(vdc_t *vdc, uint32 *target, int enabled)
 
  if(!enabled)
  {
+  uint32 color = MK_COLOR(0x00, 0xFE, 0x00) | amask;
+
   for(int x = start; x < end; x++)
-   target[x] = MK_COLOR(0x00,0xFE,0x00) | amask;
+   target[x] = color;
   return;
  }
 
@@ -858,8 +926,10 @@ static void DrawBG(vdc_t *vdc, uint32 *target, int enabled)
   else
    color = vce.color_table_cache[0x000];
 
-  for(int x = start; x < end; x++)
-   target[x] = color;
+  MDFN_FastU32MemsetM8(target, color, 576);
+
+  //for(int x = start; x < end; x++)
+  // target[x] = color;
   return;
  }
 
@@ -892,9 +962,12 @@ static void DrawBG(vdc_t *vdc, uint32 *target, int enabled)
    int palette_index = ((bat >> 12) & 0x0F) << 4;
    uint32 raw_pixel;
 
-   raw_pixel = vdc->bg_tile_cache[bat & 0x7FF][vdc->BG_YOffset & 7][vdc->BG_XOffset & 0x7] & dohmask;
+   raw_pixel = vdc->bg_tile_cache[bat & VRAM_BGTileNoMask][vdc->BG_YOffset & 7][vdc->BG_XOffset & 0x7] & dohmask;
    target[x] = vce.color_table_cache[palette_index | raw_pixel];
 
+   if((bat & 0xFFF) > VRAM_BGTileNoMask)
+    VDC_UNDEFINED("Unmapped BG tile read");
+
    vdc->BG_XOffset++;
   }
 
@@ -907,7 +980,11 @@ static void DrawBG(vdc_t *vdc, uint32 *target, int enabled)
    {
     uint16 bat = vdc->VRAM[bat_boom | bat_y];
     uint32 *lut = &vce.color_table_cache[((bat >> 8) & 0xF0)];
-    uint8 *pix_lut = vdc->bg_tile_cache[bat & 0x7FF][line_sub];
+    uint8 *pix_lut = vdc->bg_tile_cache[bat & VRAM_BGTileNoMask][line_sub];
+
+    if((bat & 0xFFF) > VRAM_BGTileNoMask)
+     VDC_UNDEFINED("Unmapped BG tile read");
+
 
     (target + 0)[x] = lut[pix_lut[0] & dohmask];
     (target + 1)[x] = lut[pix_lut[1] & dohmask];
@@ -929,7 +1006,10 @@ static void DrawBG(vdc_t *vdc, uint32 *target, int enabled)
   {
    uint16 bat = vdc->VRAM[bat_boom | bat_y];
    uint32 *lut = &vce.color_table_cache[((bat >> 8) & 0xF0)];
-   uint8 *pix_lut = vdc->bg_tile_cache[bat & 0x7FF][line_sub];
+   uint8 *pix_lut = vdc->bg_tile_cache[bat & VRAM_BGTileNoMask][line_sub];
+
+   if((bat & 0xFFF) > VRAM_BGTileNoMask)
+    VDC_UNDEFINED("Unmapped BG tile read");
 
    #ifdef LSB_FIRST
     #if SIZEOF_LONG == 8
@@ -1015,7 +1095,7 @@ static void DrawSprites(vdc_t *vdc, uint32 *target, int enabled)
  {
   int16 y = (vdc->SAT[i * 4 + 0] & 0x3FF) - 0x40;
   uint16 x = (vdc->SAT[i * 4 + 1] & 0x3FF);
-  uint16 no = (vdc->SAT[i * 4 + 2] >> 1) & 0x1FF;	// Todo, cg mode bit
+  uint16 no = (vdc->SAT[i * 4 + 2] >> 1) & 0x3FF;	// Todo, cg mode bit
   uint16 flags = (vdc->SAT[i * 4 + 3]);
 
   uint32 palette_index = (flags & 0xF) << 4;
@@ -1036,7 +1116,7 @@ static void DrawSprites(vdc_t *vdc, uint32 *target, int enabled)
     {
      vdc->status |= VDCS_OR;
      HuC6280_IRQBegin(MDFN_IQIRQ1);
-     //puts("OR IRQ");
+     VDC_DEBUG("Overflow IRQ");
     }
     if(!unlimited_sprites)
      break;
@@ -1061,34 +1141,42 @@ static void DrawSprites(vdc_t *vdc, uint32 *target, int enabled)
     SpriteList[active_sprites].x = x;
     SpriteList[active_sprites].palette_index = palette_index;
 
-
-    if((vdc->MWR & 0xC) == 4)
+    if((no * 64) >= VRAM_Size)
     {
-     if(vdc->SAT[i * 4 + 2] & 1)
+     VDC_UNDEFINED("Unmapped VRAM sprite tile read");
+     SpriteList[active_sprites].pattern_data[0] = 0;
+     SpriteList[active_sprites].pattern_data[1] = 0;
+     SpriteList[active_sprites].pattern_data[2] = 0;
+     SpriteList[active_sprites].pattern_data[3] = 0;
+    }
+    else 
+    {
+     if((vdc->MWR & 0xC) == 4)
      {
-      SpriteList[active_sprites].pattern_data[2] = 0; //vdc->VRAM[no * 64 + (y_offset & 15) ];
-      SpriteList[active_sprites].pattern_data[3] = 0; //vdc->VRAM[no * 64 + (y_offset & 15) + 16];
-      SpriteList[active_sprites].pattern_data[0] = vdc->VRAM[no * 64 + (y_offset & 15) + 32];
-      SpriteList[active_sprites].pattern_data[1] = vdc->VRAM[no * 64 + (y_offset & 15) + 48];
+      if(vdc->SAT[i * 4 + 2] & 1)
+      {
+       SpriteList[active_sprites].pattern_data[0] = vdc->VRAM[no * 64 + (y_offset & 15) + 32];
+       SpriteList[active_sprites].pattern_data[1] = vdc->VRAM[no * 64 + (y_offset & 15) + 48];
+       SpriteList[active_sprites].pattern_data[2] = 0; 
+       SpriteList[active_sprites].pattern_data[3] = 0;
+      }
+      else
+      {
+       SpriteList[active_sprites].pattern_data[0] = vdc->VRAM[no * 64 + (y_offset & 15) ];
+       SpriteList[active_sprites].pattern_data[1] = vdc->VRAM[no * 64 + (y_offset & 15) + 16];
+       SpriteList[active_sprites].pattern_data[2] = 0;
+       SpriteList[active_sprites].pattern_data[3] = 0;
+      }
      }
      else
      {
       SpriteList[active_sprites].pattern_data[0] = vdc->VRAM[no * 64 + (y_offset & 15) ];
       SpriteList[active_sprites].pattern_data[1] = vdc->VRAM[no * 64 + (y_offset & 15) + 16];
-      SpriteList[active_sprites].pattern_data[2] = 0; //vdc->VRAM[no * 64 + (y_offset & 15) + 32];
-      SpriteList[active_sprites].pattern_data[3] = 0; //vdc->VRAM[no * 64 + (y_offset & 15) + 48];
+      SpriteList[active_sprites].pattern_data[2] = vdc->VRAM[no * 64 + (y_offset & 15) + 32];
+      SpriteList[active_sprites].pattern_data[3] = vdc->VRAM[no * 64 + (y_offset & 15) + 48];
      }
     }
-    else
-    {
-	//if(y_offset == 0)
-        //printf("%d %d\n", vdc->RCRCount, no * 64);
 
-     SpriteList[active_sprites].pattern_data[0] = vdc->VRAM[no * 64 + (y_offset & 15) ];
-     SpriteList[active_sprites].pattern_data[1] = vdc->VRAM[no * 64 + (y_offset & 15) + 16];
-     SpriteList[active_sprites].pattern_data[2] = vdc->VRAM[no * 64 + (y_offset & 15) + 32];
-     SpriteList[active_sprites].pattern_data[3] = vdc->VRAM[no * 64 + (y_offset & 15) + 48];
-    }
     SpriteList[active_sprites].flags |= i ? 0 : SPRF_SPRITE0;
 
     active_sprites++;
@@ -1148,7 +1236,7 @@ static void DrawSprites(vdc_t *vdc, uint32 *target, int enabled)
     if(sprite_line_buf[tx] & (amask << 2))
     {
      vdc->status |= VDCS_CR;
-     //puts("CR IRQ");
+     VDC_DEBUG("Sprite hit IRQ");
      HuC6280_IRQBegin(MDFN_IQIRQ1);
     }
     sprite_line_buf[tx] = vce.color_table_cache[pi | raw_pixel] | prio_or;
@@ -1296,18 +1384,21 @@ void VDC_RunFrame(uint32 *pXBuf, MDFN_Rect *LineWidths, int skip)
      vdc->SATBPending = 0;
      vdc->sat_dma_slcounter = 2;
 
-     if(vdc->SATB < 0x8000)
+     if(vdc->SATB > (VRAM_Size - 0x100))
+      VDC_UNDEFINED("Unmapped VRAM SATB DMA read");
+
+     if(vdc->SATB < VRAM_Size)
      {
       uint32 len = 256;
-      if(vdc->SATB > 0x7F00)
-       len = 0x8000 - vdc->SATB;
+      if(vdc->SATB > (VRAM_Size - 0x100)) //0x7F00)
+       len = VRAM_Size - vdc->SATB;
       memcpy(vdc->SAT, &vdc->VRAM[vdc->SATB], len * sizeof(uint16));
      }
     }
    }
    if((int)vdc->RCRCount == ((int)vdc->RCR - 0x40) && (vdc->CR & 0x04))
    {
-    //printf("RCR Interrupt: %d, %02x\n", vdc->display_counter, vdc->status);
+    VDC_DEBUG("RCR IRQ");
     vdc->status |= VDCS_RR;
     HuC6280_IRQBegin(MDFN_IQIRQ1); 
    }
@@ -1388,7 +1479,10 @@ void VDC_RunFrame(uint32 *pXBuf, MDFN_Rect *LineWidths, int skip)
 
   for(int chip = 0; chip < TotalChips; chip++)
    if(vdc_chips[chip]->status & VDCS_VD)
-    HuC6280_IRQBegin(MDFN_IQIRQ1);
+   {
+    VDC_DEBUG("VBlank IRQ");
+    HuC6280_IRQBegin(MDFN_IQIRQ1);   
+   }
 
   ContinuePoint = 3;
   HuC6280_Run(455 - line_leadin1 - 2);
@@ -1410,6 +1504,7 @@ void VDC_RunFrame(uint32 *pXBuf, MDFN_Rect *LineWidths, int skip)
     {
      if(vdc->DCR & 0x01)
      {
+      VDC_DEBUG("Sprite DMA IRQ");
       vdc->status |= VDCS_DS;
       HuC6280_IRQBegin(MDFN_IQIRQ1);
      }
@@ -1537,15 +1632,15 @@ void VDC_Init(int sgx)
  }
 
  #ifdef WANT_DEBUGGER
- PCE_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vce", "VCE Palette RAM", 10);
+ MDFNDBG_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vce", "VCE Palette RAM", 10);
 
  if(sgx)
  {
-  PCE_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vdcvram0", "VDC-A VRAM", 16);
-  PCE_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vdcvram1", "VDC-B VRAM", 16);
+  MDFNDBG_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vdcvram0", "VDC-A VRAM", 16);
+  MDFNDBG_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vdcvram1", "VDC-B VRAM", 16);
  }
  else
-  PCE_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vdcvram0", "VDC VRAM", 16);
+  MDFNDBG_AddASpace(VDC_GetAddressSpaceBytes, VDC_PutAddressSpaceBytes, "vdcvram0", "VDC VRAM", 16);
  #endif
 }
 
@@ -1554,7 +1649,7 @@ void VDC_Close(void)
  for(int chip = 0; chip < TotalChips; chip++)
  {
   if(vdc_chips[chip])
-   free(vdc_chips[chip]);
+   MDFN_free(vdc_chips[chip]);
   vdc_chips[chip] = NULL;
  }
  TotalChips = 0;
@@ -1629,7 +1724,7 @@ int VDC_StateAction(StateMem *sm, int load, int data_only)
         SFVARN(vdc->status, "status"),
         SFARRAY16N(vdc->SAT, 0x100, "SAT"),
 
-        SFARRAY16N(vdc->VRAM, 32768, "VRAM"),
+        SFARRAY16N(vdc->VRAM, VRAM_Size, "VRAM"),
         SFVARN(vdc->DMAReadBuffer, "DMAReadBuffer"),
         SFVARN(vdc->DMAReadWrite, "DMAReadWrite"),
         SFVARN(vdc->DMARunning, "DMARunning"),
@@ -1659,7 +1754,7 @@ int VDC_StateAction(StateMem *sm, int load, int data_only)
   ret &= MDFNSS_StateAction(sm, load, data_only, VDC_StateRegs, chip ? "VDC1" : "VDC0");
   if(load)
   {
-   for(int x = 0; x < 32768; x++)
+   for(int x = 0; x < VRAM_Size; x++)
     FixTileCache(vdc, x);
    for(int x = 0; x < 512; x++)
     FixPCache(x);
