@@ -17,15 +17,14 @@
 
 #include <math.h>
 #include <string.h>
-#include <stdlib.h>
 #include "pcfx.h"
 #include "soundbox.h"
 #include "v810_cpu.h"
 #include "vdc.h"
 #include "king.h"
-#include "../scsicd.h"
+#include "../cdrom/scsicd.h"
 #include "timer.h"
-#include "pad.h"
+#include "input.h"
 
 typedef Blip_Synth<blip_good_quality, 65536> ADSynth;
 static ADSynth ADPCMSynth[2][2]; // [Ch0, Ch1][Left, right]
@@ -217,7 +216,7 @@ int SoundBox_Init(void)
     int x;
 
     #ifdef WANT_DEBUGGER
-    PCFX_AddRegGroup(&SBoxRegsGroup);
+    MDFNDBG_AddRegGroup(&SBoxRegsGroup);
     #endif
 
     memset(&psg, 0, sizeof(psg));
@@ -575,7 +574,7 @@ void SoundBox_Update(void)
  psg.lastts = timestamp;
 }
 
-int16 *SoundBox_Flush(int32 *len, int needrew)
+int16 *SoundBox_Flush(int32 *len)
 {
  int32 timestamp;
 
@@ -586,7 +585,7 @@ int16 *SoundBox_Flush(int32 *len, int needrew)
  DoADPCMUpdate();
  SCSICD_Run(v810_timestamp);
  FXTIMER_Update();
- FXPAD_Update();
+ FXINPUT_Update();
 
  timestamp = v810_timestamp / 3;
  if(FSettings.SndRate)
@@ -600,13 +599,17 @@ int16 *SoundBox_Flush(int32 *len, int needrew)
   *len = love;
  }
 
+ KING_ResetTS(); // Call it before setting v810_timestamp to 0
+
+ pcfx_timestamp_base += v810_timestamp;
  v810_timestamp = 0;
+
  psg.lastts = 0;
  adpcm_lastts = 0;
 
  SCSICD_ResetTS();
  FXTIMER_ResetTS();
- FXPAD_ResetTS();
+ FXINPUT_ResetTS();
  
  if(!FSettings.SndRate)
  {
