@@ -196,19 +196,30 @@ static const char *pretty_preg_names[32] =
 };
 
 
-void v810_dis(uint32 &tPC, int num, char *buf)
+void v810_dis(uint32 &tPC, int num, char *buf, uint16 (*rhword)(uint32))
 {
     int lowB, highB, lowB2, highB2;             // up to 4 bytes for instruction (either 16 or 32 bits)
     int opcode, arg1, arg2, arg3;
     int i = 0;
 
     buf[0] = 0;
-    for(i = 0; i< num; i++) {
-        lowB   = mem_rbyte(tPC);
-        highB  = mem_rbyte(tPC+1);
-        lowB2  = mem_rbyte(tPC+2);
-        highB2 = mem_rbyte(tPC+3);
-    
+
+    for(i = 0; i< num; i++) 
+    {
+     uint16 tmp;
+
+     tmp = rhword(tPC);
+     //   lowB   = mem_rbyte(tPC);
+     //   highB  = mem_rbyte(tPC+1);
+     lowB = tmp & 0xFF;
+     highB = tmp >> 8;
+
+     tmp = rhword(tPC + 2);
+     //   lowB2  = mem_rbyte(tPC+2);
+     //   highB2 = mem_rbyte(tPC+3);
+     lowB2 = tmp & 0xFF;
+     highB2 = tmp >> 8;
+
         opcode = highB >> 2;
         if((highB & 0xE0) == 0x80)        // Special opcode format for          
             opcode = (highB >> 1);            // type III instructions.
