@@ -5,14 +5,16 @@
 #include "mednafen.h"
 #include "settings.h"
 #include "md5.h"
-#include "world_strtod.h"
+#include "string/world_strtod.h"
 
 std::vector <MDFNCS> CurrentSettings;
 static std::string fname;
 
 static bool ValidateSetting(const char *value, const MDFNSetting *setting)
 {
- if(setting->type == MDFNST_UINT)
+ MDFNSettingType base_type = (MDFNSettingType)(setting->type & MDFNST_BASE_MASK);
+
+ if(base_type == MDFNST_UINT)
  {
   unsigned long long ullvalue;
   char *endptr = NULL;
@@ -46,7 +48,7 @@ static bool ValidateSetting(const char *value, const MDFNSetting *setting)
    }
   }
  }
- else if(setting->type == MDFNST_INT)
+ else if(base_type == MDFNST_INT)
  {
   long long llvalue;
   char *endptr = NULL;
@@ -81,7 +83,7 @@ static bool ValidateSetting(const char *value, const MDFNSetting *setting)
    }
   }
  }
- else if(setting->type == MDFNST_FLOAT)
+ else if(base_type == MDFNST_FLOAT)
  {
   double dvalue;
   char *endptr = NULL;
@@ -116,7 +118,7 @@ static bool ValidateSetting(const char *value, const MDFNSetting *setting)
    }
   }
  }
- else if(setting->type == MDFNST_BOOL)
+ else if(base_type == MDFNST_BOOL)
  {
   if(strlen(value) != 1 || (value[0] != '0' && value[0] != '1'))
   {
@@ -127,7 +129,7 @@ static bool ValidateSetting(const char *value, const MDFNSetting *setting)
 
  if(setting->validate_func && !setting->validate_func(setting->name, value))
  {
-  if(setting->type == MDFNST_STRING)
+  if(base_type == MDFNST_STRING)
    MDFN_PrintError(_("Setting \"%s\" is not set to a valid string: \"%s\""), setting->name, value);
   else
    MDFN_PrintError(_("Setting \"%s\" is not set to a valid unsigned integer: \"%s\""), setting->name, value);
@@ -227,11 +229,14 @@ static INLINE void MergeSettingSub(const MDFNSetting *setting)
    name_hash |= md5out[0] << (x * 8);
 
   MDFNCS TempSetting;
+
   TempSetting.name = strdup(setting->name);
   TempSetting.value = strdup(setting->default_value);
   TempSetting.name_hash = name_hash;
   TempSetting.desc = setting;
   TempSetting.ChangeNotification = setting->ChangeNotification;
+  TempSetting.netplay_override = NULL;
+
   CurrentSettings.push_back(TempSetting);
 }
 
@@ -272,6 +277,8 @@ bool MDFN_SaveSettings(void)
  if(!(fp = fopen(fname.c_str(), "wb")))
   return(0);
 
+ fprintf(fp, ";VERSION %s\n", MEDNAFEN_VERSION);
+
  fprintf(fp, _(";Edit this file at your own risk!\n"));
  fprintf(fp, _(";File format: <key><single space><value><LF or CR+LF>\n\n"));
  for(unsigned int x = 0; x < CurrentSettings.size(); x++)
@@ -305,7 +312,10 @@ static const char *FindSetting(const char *name)
  {
   if(CurrentSettings[x].name_hash == name_hash && !strcmp(CurrentSettings[x].name, name))
   {
-   value = CurrentSettings[x].value;
+   if(CurrentSettings[x].netplay_override)
+    value = CurrentSettings[x].netplay_override;
+   else
+    value = CurrentSettings[x].value;
   }
  }
  if(!value)
@@ -346,7 +356,7 @@ const std::vector <MDFNCS> *MDFNI_GetSettings(void)
  return(&CurrentSettings);
 }
 
-bool MDFNI_SetSetting(const char *name, const char *value)
+bool MDFNI_SetSetting(const char *name, const char *value, bool NetplayOverride)
 {
  md5_context md5;
  uint8 md5out[16];
@@ -367,8 +377,19 @@ bool MDFNI_SetSetting(const char *name, const char *value)
    {
     return(0);
    }
-   if(CurrentSettings[x].value) free(CurrentSettings[x].value);
-   CurrentSettings[x].value = strdup(value);
+
+   // TODO:  When NetplayOverride is set, make sure the setting is an emulation-related setting, 
+   // and that it is safe to change it(changing paths to BIOSes and such is not safe :b).
+   if(NetplayOverride)
+   {
+    if(CurrentSettings[x].netplay_override) free(CurrentSettings[x].netplay_override);
+    CurrentSettings[x].netplay_override = strdup(value);
+   }
+   else
+   {
+    if(CurrentSettings[x].value) free(CurrentSettings[x].value);
+    CurrentSettings[x].value = strdup(value);
+   }
 
    // TODO, always call driver notification function, regardless of whether a game is loaded.
    if(CurrentSettings[x].ChangeNotification)
@@ -385,13 +406,29 @@ bool MDFNI_SetSetting(const char *name, const char *value)
  return(0);
 }
 
+#if 0
+// TODO after a game is loaded, but should we?
+void MDFN_CallSettingsNotification(void)
+{
+ for(unsigned int x = 0; x < CurrentSettings.size(); x++)
+ {
+  if(CurrentSettings[x].ChangeNotification)
+  {
+   // TODO, always call driver notification function, regardless of whether a game is loaded.
+   if(MDFNGameInfo)
+    CurrentSettings[x].ChangeNotification(CurrentSettings[x].name);
+  }
+ }
+}
+#endif
+
 bool MDFNI_SetSettingB(const char *name, bool value)
 {
  char tmpstr[2];
  tmpstr[0] = value ? '1' : '0';
  tmpstr[1] = 0;
 
- return(MDFNI_SetSetting(name, tmpstr));
+ return(MDFNI_SetSetting(name, tmpstr, FALSE));
 }
 
 bool MDFNI_SetSettingUI(const char *name, uint32 value)
@@ -399,6 +436,6 @@ bool MDFNI_SetSettingUI(const char *name, uint32 value)
  char tmpstr[32];
 
  sprintf(tmpstr, "%lu", (long)value);
- return(MDFNI_SetSetting(name, tmpstr));
+ return(MDFNI_SetSetting(name, tmpstr, FALSE));
 }
 
