@@ -22,6 +22,8 @@ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
+#include "../sexyal.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -36,9 +38,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <sched.h>
 #include <sys/soundcard.h>
 
-#include "../sexyal.h"
-#include "../smallc.h"
-
 #include "oss.h"
 
 SexyAL_enumdevice *SexyALI_OSS_EnumerateDevices(void)
@@ -46,6 +45,7 @@ SexyAL_enumdevice *SexyALI_OSS_EnumerateDevices(void)
  SexyAL_enumdevice *ret,*tmp,*last;
  struct stat buf;
  char fn[64];
+ char numstring[64];
  unsigned int n;
 
  n=0;
@@ -54,15 +54,16 @@ SexyAL_enumdevice *SexyALI_OSS_EnumerateDevices(void)
 
  for(;;)
  {
-  strcpy(fn,"/dev/dsp");
-  strcat(fn,sal_uinttos(n));
+  snprintf(numstring, 64, "%d", n);
+  snprintf(fn, 64, "/dev/dsp%s", numstring);
+
   if(stat(fn,&buf)!=0) break;
 
   tmp = malloc(sizeof(SexyAL_enumdevice));
   memset(tmp, 0, sizeof(SexyAL_enumdevice));
 
   tmp->name = strdup(fn);
-  tmp->id = strdup(sal_uinttos(n));
+  tmp->id = strdup(numstring);
 
   if(!ret) ret = tmp;
   if(last) last->next = tmp;
