@@ -15,15 +15,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include "mednafen.h"
+
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "types.h"
-#include "mednafen.h"
 #include "driver.h"
 #include "endian.h"
 #include "general.h"
@@ -37,7 +35,6 @@ static int SaveStateStatus[10];
 
 #define RLSB 		MDFNSTATE_RLSB	//0x80000000
 
-
 int32 smem_read(StateMem *st, void *buffer, uint32 len)
 {
  if((len + st->loc) > st->len)
@@ -187,31 +184,31 @@ static int SubWrite(StateMem *st, SFORMAT *sf, int data_only, gzFile fp)
   //printf("%d %d %d\n", bytesize, data_only, fp);
   if(st || fp)			/* Are we writing or calculating the size of this block? */
   {
-   char desco[32];
-   int slen = strlen(sf->desc);
-
-   memset(desco, 0, 32);
-
-   if(slen > 32) 
-   {
-    printf("Warning:  state variable name too long: %s %d\n", sf->desc, slen);
-    slen = 32;
-   }
-   memcpy(desco, sf->desc, slen);
-
    if(!data_only)
    {
+    char desco[32];
+    int slen = strlen(sf->desc);
+
+    memset(desco, 0, 32);
+
+    if(slen > 32)
+    {
+     printf("Warning:  state variable name too long: %s %d\n", sf->desc, slen);
+     slen = 32;
+    }
+
+    memcpy(desco, sf->desc, slen);
     smem_write(st, desco, 32);
     smem_write32le(st, bytesize);
-   }
 
-   /* Flip the byte order... */
-   if(sf->s & MDFNSTATE_RLSB32)
-    Endian_A32_NE_to_LE(sf->v, bytesize / sizeof(uint32));
-   else if(sf->s & MDFNSTATE_RLSB16)
-    Endian_A16_NE_to_LE(sf->v, bytesize / sizeof(uint16));
-   else if(sf->s&RLSB)
-    Endian_V_NE_to_LE(sf->v, bytesize);
+    /* Flip the byte order... */
+    if(sf->s & MDFNSTATE_RLSB32)
+     Endian_A32_NE_to_LE(sf->v, bytesize / sizeof(uint32));
+    else if(sf->s & MDFNSTATE_RLSB16)
+     Endian_A16_NE_to_LE(sf->v, bytesize / sizeof(uint16));
+    else if(sf->s&RLSB)
+     Endian_V_NE_to_LE(sf->v, bytesize);
+   }
 
    if(fp)
    {
@@ -222,13 +219,17 @@ static int SubWrite(StateMem *st, SFORMAT *sf, int data_only, gzFile fp)
    {
     smem_write(st, (uint8 *)sf->v, bytesize);
    }
-   /* Now restore the original byte order. */
-   if(sf->s & MDFNSTATE_RLSB32)
-    Endian_A32_LE_to_NE(sf->v, bytesize / sizeof(uint32));
-   else if(sf->s & MDFNSTATE_RLSB16)
-    Endian_A16_LE_to_NE(sf->v, bytesize / sizeof(uint16));
-   else if(sf->s&RLSB)
-    Endian_V_LE_to_NE(sf->v, bytesize);
+
+   if(!data_only)
+   {
+    /* Now restore the original byte order. */
+    if(sf->s & MDFNSTATE_RLSB32)
+     Endian_A32_LE_to_NE(sf->v, bytesize / sizeof(uint32));
+    else if(sf->s & MDFNSTATE_RLSB16)
+     Endian_A16_LE_to_NE(sf->v, bytesize / sizeof(uint16));
+    else if(sf->s&RLSB)
+     Endian_V_LE_to_NE(sf->v, bytesize);
+   }
   }
   sf++; 
  }
@@ -369,7 +370,6 @@ static int ReadStateChunk(StateMem *st, SFORMAT *sf, int size, int data_only)
 }
 
 int CurrentState = 0;
-extern int geniestage;
 
 static void MakeStatePreview(uint8 *dest, uint32 *fb, MDFN_Rect *LineWidths)
 {
@@ -556,6 +556,13 @@ int MDFNSS_SaveSM(StateMem *st, int wantpreview, int data_only, uint32 *fb, MDFN
 	 free(previewbuffer);
 	}
 
+        // State rewinding code path hack, FIXME
+        if(data_only)
+        {
+         if(!MDFN_RawInputStateAction(st, 0, data_only))
+          return(0);
+        }
+
 	if(!MDFNGameInfo->StateAction(st, 0, data_only))
 	 return(0);
 
@@ -570,45 +577,68 @@ int MDFNSS_SaveSM(StateMem *st, int wantpreview, int data_only, uint32 *fb, MDFN
 
 int MDFNSS_Save(const char *fname, const char *suffix, uint32 *fb, MDFN_Rect *LineWidths)
 {
-	gzFile st = NULL;
+	StateMem st;
 
-	if(geniestage==1)
-	{
-	 MDFN_DispMessage((UTF8 *)_("Cannot use states in GG Screen."));
-	 return(0);
-        }
+	memset(&st, 0, sizeof(StateMem));
 
-	if(fname)
-	 st=gzopen(fname, "wb6");
-	else
-	{
-         st=gzopen(MDFN_MakeFName(MDFNMKF_STATE,CurrentState,suffix).c_str(),"wb6");
-	}
-
-	if(st == NULL)
+	if(!MDFNSS_SaveSM(&st, 1, 0, fb, LineWidths))
 	{
+	 if(st.data)
+	  free(st.data);
 	 if(!fname && !suffix)
-          MDFN_DispMessage((UTF8 *)_("State %d save error."),CurrentState);
+ 	  MDFN_DispMessage((UTF8 *)_("State %d save error."), CurrentState);
 	 return(0);
 	}
 
-	if(!MDFNSS_SaveFP(st, fb, LineWidths))
-        {
+	if(!MDFN_DumpToFile(fname ? fname : MDFN_MakeFName(MDFNMKF_STATE,CurrentState,suffix).c_str(), 6, st.data, st.len))
+	{
          SaveStateStatus[CurrentState] = 0;
-         gzclose(st);
+	 free(st.data);
+
          if(!fname && !suffix)
           MDFN_DispMessage((UTF8 *)_("State %d save error."),CurrentState);
+
 	 return(0);
-        }
+	}
+
+	free(st.data);
 
 	SaveStateStatus[CurrentState] = 1;
-	gzclose(st);
 
 	if(!fname && !suffix)
 	 MDFN_DispMessage((UTF8 *)_("State %d saved."),CurrentState);
+
 	return(1);
 }
 
+// Convenience function for movie.cpp
+int MDFNSS_SaveFP(gzFile fp, uint32 *fb, MDFN_Rect *LineWidths)
+{
+ StateMem st;
+
+ memset(&st, 0, sizeof(StateMem));
+
+ if(!MDFNSS_SaveSM(&st, 1, 0, fb, LineWidths))
+ {
+  if(st.data)
+   free(st.data);
+  return(0);
+ }
+
+ if(gzwrite(fp, st.data, st.len) != (int32)st.len)
+ {
+  if(st.data)
+   free(st.data);
+  return(0);
+ }
+
+ if(st.data)
+  free(st.data);
+
+ return(1);
+}
+
+
 int MDFNSS_LoadSM(StateMem *st, int haspreview, int data_only)
 {
         uint8 header[32];
@@ -642,6 +672,14 @@ int MDFNSS_LoadSM(StateMem *st, int haspreview, int data_only)
 	 psize = width * height * 3;
 	 smem_seek(st, psize, SEEK_CUR);	// Skip preview
  	}
+
+	// State rewinding code path hack, FIXME
+	if(data_only)
+	{
+	 if(!MDFN_RawInputStateAction(st, stateversion, data_only))
+	  return(0);
+	}
+
 	return(MDFNGameInfo->StateAction(st, stateversion, data_only));
 }
 
@@ -679,43 +717,10 @@ int MDFNSS_LoadFP(gzFile fp)
  return(1);
 }
 
-
-int MDFNSS_SaveFP(gzFile fp, uint32 *fb, MDFN_Rect *LineWidths)
-{
- StateMem st;
-
- memset(&st, 0, sizeof(StateMem));
-
- if(!MDFNSS_SaveSM(&st, 1, 0, fb, LineWidths))
- {
-  if(st.data)
-   free(st.data);
-  return(0);
- }
-
- if(gzwrite(fp, st.data, st.len) != (int32)st.len)
- {
-  if(st.data)
-   free(st.data);
-  return(0);
- }
-
- if(st.data)
-  free(st.data);
-
- return(1);
-}
-
 int MDFNSS_Load(const char *fname, const char *suffix)
 {
 	gzFile st;
 
-        if(geniestage==1)
-        {
-         MDFN_DispMessage((UTF8 *)_("Cannot use states in GG Screen."));
-         return(0);
-        }
-
         if(fname)
          st=gzopen(fname, "rb");
         else
@@ -893,13 +898,15 @@ void MDFNI_LoadState(const char *fname, const char *suffix)
  doothack = 100;
 }
 
-#include "minilzo.h"
-#include "quicklz.h"
+#include "compress/minilzo.h"
+#include "compress/quicklz.h"
+#include "compress/blz.h"
 
 enum
 {
  SRW_COMPRESSOR_MINILZO = 0,
- SRW_COMPRESSOR_QUICKLZ
+ SRW_COMPRESSOR_QUICKLZ,
+ SRW_COMPRESSOR_BLZ
 };
 
 typedef struct
@@ -911,8 +918,7 @@ typedef struct
 	StateMem MovieLove;
 } StateMemPacket;
 
-#define SRW_NUM		600
-
+static int SRW_NUM = 600;
 static int SRWCompressor;
 static int EvilEnabled = 0;
 static StateMemPacket *bcs;
@@ -926,6 +932,8 @@ void MDFN_StateEvilBegin(void)
  if(!EvilEnabled)
   return;
 
+ SRW_NUM = MDFN_GetSettingUI("srwframes");
+
  SRWCompressor = SRW_COMPRESSOR_MINILZO;
  srwcompstring = MDFN_GetSettingS("srwcompressor");
 
@@ -933,6 +941,8 @@ void MDFN_StateEvilBegin(void)
   SRWCompressor = SRW_COMPRESSOR_MINILZO;
  else if(srwcompstring == "quicklz")
   SRWCompressor  = SRW_COMPRESSOR_QUICKLZ;
+ else if(srwcompstring == "blz")
+  SRWCompressor = SRW_COMPRESSOR_BLZ;
 
  bcs = (StateMemPacket *)calloc(SRW_NUM, sizeof(StateMemPacket));
  bcspos = 0;
@@ -1018,9 +1028,12 @@ int MDFN_StateEvil(int rewind)
 
    if(SRWCompressor == SRW_COMPRESSOR_QUICKLZ)
     dst_len = qlz_decompress((char*)bcs[bcspos].data, tmp_buf);
-   else
+   else if(SRWCompressor == SRW_COMPRESSOR_MINILZO)
     lzo1x_decompress(bcs[bcspos].data, bcs[bcspos].compressed_len, tmp_buf, &dst_len, NULL);
-
+   else if(SRWCompressor == SRW_COMPRESSOR_BLZ)
+   {
+    dst_len = blz_unpack(bcs[bcspos].data, tmp_buf);
+   }
    for(uint32 x = 0; x < bcs[bcspos].uncompressed_len && x < bcs[next_bcspos].uncompressed_len; x++)
     tmp_buf[x] ^= bcs[next_bcspos].data[x];
 
@@ -1059,7 +1072,6 @@ int MDFN_StateEvil(int rewind)
  }
  else
  {
-  uint8 workmem[LZO1X_1_MEM_COMPRESS];
   StateMem sm;
   int32 prev_bcspos = bcspos;
 
@@ -1085,7 +1097,9 @@ int MDFN_StateEvil(int rewind)
    free(bcs[bcspos].MovieLove.data);
    bcs[bcspos].MovieLove.data = NULL;
   }
+
   memset(&sm, 0, sizeof(sm));
+
   MDFNSS_SaveSM(&sm, 0, 1);
 
   bcs[bcspos].data = sm.data;
@@ -1109,13 +1123,27 @@ int MDFN_StateEvil(int rewind)
     bcs[prev_bcspos].data = (uint8 *)realloc(tmp_buf, dst_len);
     bcs[prev_bcspos].compressed_len = dst_len;
    }
-   else
+   else if(SRWCompressor == SRW_COMPRESSOR_MINILZO)
    {
+    uint8 workmem[LZO1X_1_MEM_COMPRESS];
     uint8 * tmp_buf = (uint8 *)malloc((size_t)(1.10 * bcs[prev_bcspos].uncompressed_len));
     lzo_uint dst_len = (lzo_uint)(1.10 * bcs[prev_bcspos].uncompressed_len);
 
     lzo1x_1_compress(bcs[prev_bcspos].data, bcs[prev_bcspos].uncompressed_len, tmp_buf, &dst_len, workmem);
 
+    free(bcs[prev_bcspos].data);
+    bcs[prev_bcspos].data = (uint8 *)realloc(tmp_buf, dst_len);
+    bcs[prev_bcspos].compressed_len = dst_len;
+   }
+   else if(SRWCompressor == SRW_COMPRESSOR_BLZ)
+   {
+    blz_pack_t workmem;
+
+    uint8 * tmp_buf = (uint8 *)malloc((size_t)(bcs[prev_bcspos].uncompressed_len + blz_pack_extra));
+    uint32 dst_len = bcs[prev_bcspos].uncompressed_len + blz_pack_extra;
+
+    dst_len = blz_pack(bcs[prev_bcspos].data, bcs[prev_bcspos].uncompressed_len, tmp_buf, &workmem);
+
     free(bcs[prev_bcspos].data);
     bcs[prev_bcspos].data = (uint8 *)realloc(tmp_buf, dst_len);
     bcs[prev_bcspos].compressed_len = dst_len;
